#  $Id: gist.py,v 1.17 2003/09/30 23:43:47 travo Exp $
#  ---------------------------------------------------------------------
#
#  NAME:  gist.py
#
#  SCIPY
#  09/28/03 teo Changed eps file to accept noepsi when not available.
#  09/28/03 teo Changed all == None to is None and != None to not is None
#
#  CHANGES:
#  03/13/03 llc Add one NOTE on plfc.
#  03/12/03 llc Updated doc comments.
#  12/25/02 mdh Add plh to draw histograms
#  11/26/01 llc Add docstring for plmk (missing).
#  11/05/01 llc Use pydoc's help (not the one in help.py).
#  10/30/01 llc Disable is_scalar call; type(arraytype) not implemented
#               in Python.
#               Also, merge in documentation (if it exists) from gist.help 
#               after each function, so that pydoc's help can return it.
#  10/12/01 llc Re-port of gist from archived version.
#  03/19/03 llc Dave Grote reported a bug in plfc:  nc==None should be 
#               nc is None.
#  04/07/03 mdh Modifications to plh to add labels below x-axis; removed 
#               legend and added label keyword.
#
#  ---------------------------------------------------------------------

"""
     Gist is a portable graphics package for scientific applications. It
     can produce interactive graphics for Unix/Linux (X11), Windows, 
     and Mac platforms, as well as produce file output conforming to ANSI 
     standard CGM or standard Postscript.

     Gist was developed by David H. Munro (munro1@llnl.gov) at
     Lawrence Livermore National Laboratory, as part of his Yorick
     scientific interpreter. Lee Busby was the original author of 
     the Python Gist C extension module.  He adapted much of the 
     module from similar code written by Munro for Yorick.

     Copyright (c) 1996, 1997, The Regents of the University of
     California.  All rights reserved.  See Legal.htm for full
     text and disclaimer.

     *********************************************************************

     This version of PyGist is built on the portability layer from 
     Yorick 1.5.  The documentation for each function has been 
     integrated with the function using pydoc.  Type:

        help(function_name)

     to get the inline documentation for function_name.
     Hit spacebar to page down, and 'q' to end documentation.
     Single quotes and backquotes delimiting strings in documentation
     should be double quotes.
"""

__version__ = "1.5.18"

from Numeric import *
import sys, os	# To be sure expand_path has posixpath and we have sys.path
from gistC import *
from pydoc import help
from shapetest import *
from arrayfns import *

# Parameters used by pltitle and xytitles
pltitle_height= 18;
pltitle_font= "helvetica";

# Parameters used by plmk and plmk_default
_plmk_count = 0
_plmk_msize = _plmk_color = _plmk_width = None

#  ---------------------------------------------------------------------

def winkill(*N):
   """
   winkill( [n] )
      Delete the current graphics window, or graphics window N (0-7).
   """
   if N:
      window(N[0], display="", hcp="")
   else:
      window(display="", hcp="")

#  ---------------------------------------------------------------------

def pltitle(title):
   """
   pltitle( title )
      Plot TITLE centered above the coordinate system for any of the
      standard Gist styles.  You will need to customize this for other
      plot styles.
   """
   vp = viewport()
   xmidpt = (vp[0] + vp[1])/2.0
   plt( title, xmidpt, vp[3] + 0.02,
      font=pltitle_font, justify="CB", height=pltitle_height)

#  ---------------------------------------------------------------------

def ylimits(ymin='u',ymax='u'): 
   """
   ylimits(ymin, ymax)
      Set the y-axis plot limits in the current coordinate system to
      YMIN, YMAX, which may each be a number to fix the corresponding
      limit to a specified value, or the string "e" to make the
      corresponding limit take on the extreme value of the currently
      displayed data. Arguments may be omitted only from the right. Use
      limits( xmin, xmax ) to accomplish the same function for the x-axis
      plot limits.  Note that the corresponding Yorick function for
      ylimits is ``range'' - since this word is a Python built-in function,
      I've changed the name to avoid the collision.
      SEE ALSO: plsys, limits, logxy, plg
   """
   limits('u','u',ymin,ymax)

#  ---------------------------------------------------------------------

def moush(*arg):
   """
   moush(y, x, ireg) or moush(y, x) or moush()
      Number of args can be 0, 2, or 3.
      Returns the 1-origin zone index for the point clicked in
      for the default mesh, or for the mesh (X,Y) (region array IREG).
   """
   narg = len(arg)
   if narg == 3: # (y, x, ireg)
      xy = mouse (-1, 0, "<Click mouse in mesh>")
      if xy is None: return None
      return mesh_loc (xy[1], xy[0], arg[0], arg[1], arg[2]);
   elif narg == 2: # (y, x)
      xy = mouse (-1, 0, "<Click mouse in mesh>")
      if xy is None: return None
      return mesh_loc (xy[1], xy[0], arg[0], arg[1]);
   elif narg == 0: # ()
      xy = mouse (-1, 0, "<Click mouse in mesh>")
      if xy is None: return None
      return mesh_loc (xy[1], xy[0]);
   else:
      print "Mouse takes 0, 2, or 3 args: ( [ y, x [ , ireg ] ] )"
      return None

#  ---------------------------------------------------------------------
#  PURPOSE:  Create an encapsulated PostScript file.  
#            Requires Ghostscript and its associated ps2epsi utility.
#  ---------------------------------------------------------------------

import os
import shutil
def eps(name,epsi=0,pdf=0):
   """
      Write the picture in the current graphics window to the Encapsulated
      PostScript file NAME+".eps" (i.e.- the suffix .eps is added to NAME).
      The last extension of name is stripped (if it is .eps)
      to avoid .eps.eps files

      Uses the command ps2epsi if available which comes with the project
      GNU Ghostscript program.  Any hardcopy file associated
      with the current window is first closed, but the default hardcopy file is
      unaffected.  As a side effect, legends are turned off and color table
      dumping is turned on for the current window.
      
      The environment variable PS2EPSI_FORMAT contains the format for the
      command to start the ps2epsi program.
      
      SEE ALSO: window, fma, hcp, hcp_finish, plg
   """
   name,ignore = os.path.splitext(name)
   if ignore == '.eps':
      totalname = name
   else:
      totalname = name + ignore
   apath, basename = os.path.split(totalname)
   name = totalname + ".ps"
   window (hcp = name, dump = 1, legends = 0)
   hcp ()
   window (hcp="")
   command = os.environ.get('PS2EPSI_FORMAT') or 'ps2epsi'
   os.system ("%s %s" % (command, name))
   # check for presence of epsi file to see if ps2epsi command
   #   succeeded.
   if os.path.exists('%s.epsi' % basename):
      os.remove(name)
      shutil.copy("%s.epsi" % basename, "%s.eps" % totalname)
      os.remove("%s.epsi" % basename)
   else:
      os.rename("%s.ps" % totalname, "%s.eps" % totalname)
   if pdf:
      os.system("epstopdf %s.eps" % totalname)

#  ---------------------------------------------------------------------

def xytitles(xtitle = "", ytitle = "", delta = (0.,0.)):

   vp = viewport()
   xmidpt = (vp[0] + vp[1])/2.0
   ymidpt = (vp[2] + vp[3])/2.0
   if len(xtitle) > 0:
      plt(xtitle, xmidpt, vp[2] - 0.050 + delta[1],
         font=pltitle_font, justify="CT", height=pltitle_height)
   if len(ytitle) > 0:
      plt(ytitle, vp[0] - 0.050 + delta[0], ymidpt,
         font=pltitle_font, justify="CB", height=pltitle_height, orient=1)

#  ---------------------------------------------------------------------

def _spanz(lb,ub,n):

   if n < 3: raise ValueError, '3rd arg must be at least 3'
   c = 0.5*(ub - lb)/(n - 1.0)
   b = lb + c
   a = (ub - c - b)/(n - 2.0)
   return map(lambda x,A=a,B=b: A*x + B, arange(n-1))

#  .. predefined markers: square, +, delta, circle, diamond, x, grad

_seq = _spanz(-pi,pi,37)
_plmk_markers = (
   array([[-1,1,1,-1],[-1,-1,1,1]])*.007,
   array([[-4,-1,-1,1,1,4,4,1,1,-1,-1,-4],
      [-1,-1,-4,-4,-1,-1,1,1,4,4,1,1]])*.007/sqrt(7),
   array([[-sqrt(3),sqrt(3),0],[-1,-1,2]])*.007/sqrt(.75*sqrt(3)),
   array([cos(_seq),sin(_seq)])*.007/(pi/4.),
   array([[-1,0,1,0],[0,-1,0,1]])*.007*sqrt(2),
   array([[-1,-2.5,-1.5,0,1.5,2.5,1,2.5,1.5,0,-1.5,-2.5],
      [0,-1.5,-2.5,-1,-2.5,-1.5,0,1.5,2.5,1,2.5,1.5]])*.007*sqrt(2)/sqrt(7),
   array([[0,sqrt(3),-sqrt(3)],[-2,1,1]])*.007/sqrt(.75*sqrt(3))
   )
del(_seq)

#  ---------------------------------------------------------------------

def plmk(y,x=None,marker=None,width=None,color=None,msize=None):
   """
   Make a scatter plot of the points Y versus X.  If X is nil,
   it defaults to indgen(numberof(Y)).  By default, the marker
   cycles through 7 predefined marker shapes.  You may specify a shape
   using the marker= keyword, line width using the width= keyword (you
   get solid fills for width>=10), color using the color= keyword.
   You can also use the msize= keyword to scale the marker (default
   msize=1.0).  You can change the default width, color, or msize
   using the plmk_default function.

   The predefined marker= values are:

   marker=
      1        square
      2        cross
      3        triangle
      4        circle
      5        diamond
      6        cross (rotated 45 degrees)
      7        triangle (upside down)

   You may also put marker=[xm,ym] where xm and ym are vectors
   of NDC coordinates to design your own custom marker shapes.

   SEE ALSO: plmk_default, plg (type=0 keyword)
   """

   global _plmk_count
   global _plmk_color, _plmk_width, _plmk_msize
   color_dict = { 'bg':-1, 'fg':-2, 'black':-3, 'white':-4, 'red':-5,
      'green':-6, 'blue':-7, 'cyan':-8, 'magenta':-9, 'yellow':-10 }

   z = None

   if marker is None:
      marker = _plmk_markers[(_plmk_count)%7]
      _plmk_count = _plmk_count + 1
   elif type(marker) == type(0):
      marker = _plmk_markers[marker-1];

   xm = marker[0]
   ym = marker[1]
   if not msize: msize = _plmk_msize;
   if msize:
      xm = xm * msize;
      ym = ym * msize;

   if not color: color = _plmk_color;
   ecolor = color;
   if type(color) == type(""):
      color = color_dict[color];
  
   if not width: width = _plmk_width;
   if width >= 10:
      if not color:
         color = ecolor = -2 # magic number for "fg"
      z = ones(1+len(y)) * color
      z = z.astype(UnsignedInt8) # convert array to type <unsigned char>
      width = None

   n = ones(1+len(y));
   n[0] = len(ym);
   if not x: x = 1 + arange(len(y));
   plfp( z, concatenate((ym,y)), concatenate((xm,x)),n,
      edges=1, ewidth=width, ecolor=ecolor)

#  ---------------------------------------------------------------------


def plmk_default(color=None, msize=None, width=None):
   """
   Set default color, msize, and width values for plmk.  Use
   width=10 to get solid fills.  With no parameters, plmk_default
   restores the initial default values.
   """
   global _plmk_color, _plmk_width, _plmk_msize
   if color: _plmk_color = color
   if width: _plmk_width = width
   if msize: _plmk_msize = msize
   if not (color or width or msize):
      _plmk_msize = _plmk_color = _plmk_width = None

from arrayfns import *
from types import *

#  ---------------------------------------------------------------------

def spann (zmin, zmax, n = 8, fudge = 0, force = 0) :
   """
   spann (zmin, zmax, n = 8, fudge = 0, force = 0)
      return no more than N equally spaced "nice" numbers between
      ZMIN and ZMAX.
      Note that in general spann may not supply the number of
      values that you asked for. To force it to do so, set
      keyword FORCE to nonzero.
      SEE ALSO: span, spanl, plc, plfc
   """
   dz = (zmax - zmin) / max (float (n), 0.)
   if dz == 0. :
      dz = abs (zmin)
   if (dz != 0.) :
      power = floor (log10 (dz) + 0.00001)
      base = dz / (10. ** power)
      if base > 5.00001 :
         base = 1.0
         power = power + 1.0
      elif base > 2.00001 :
         base = 5.0
      else :
         base = 2.0
      # Round dz up to nearest "nice" number
      dz = base * 10.0 ** power
      zmin = ceil (zmin / dz - fudge)
      zmax = floor (zmax / dz + fudge)
      if (force == 0) :
         nz = int (zmax - zmin + 1.0)
      else :
         nz = n
      if nz > 1 :
         levs = span (zmin * dz, zmax * dz, nz)
      else :
         if nz < 1 :
            if base < 1.5 :
               base = 5.0
               power = power - 1
            elif base < 2.5 :
               base = 1.0
            else :
               base = 2.0
            dz = base * 10.0 ** power
            zmin = ceil (zmin / dz + 0.001)
         levs = array ( [zmin * dz])
   else :
      levs = array ( [-1.0, 1.0])
   return (levs)

_ContourError = "ContourError"

#  ---------------------------------------------------------------------

def plfc (z, y, x, ireg, contours = 8, colors = None, region = 0,
   triangle = None, scale = "lin") :
   """
   plfc (z, y, x, ireg, contours = 8, colors = None, region = 0,
      triangle = None, scale = "lin")
      fills contours of Z on the mesh Y versus X.  Y, X, and IREG are
      as for plm.  The Z array must have the same shape as Y and X.
      The function being contoured takes the value Z at each point
      (X, Y) -- that is, the Z array is presumed to be point-centered.

      NOTE:  The ireg argument was not in the Yorick Gist plfc.

      The CONTOURS keyword can be an integer specifying the number of
      contours desired, or a list of the values of Z at which you want
      contour curves.  These curves divide the mesh into len(CONTOURS+1)
      regions, each of which is filled with a solid color.  If CONTOURS is
      None or not given, 8 "nice" equally spaced level values spanning the
      range of Z are selected.

      If you specify CONTOURS, you may also specify COLORS, an array of
      color numbers (Python typecode 'b', integers between 0 and the
      length of the current palette - 1, normally 199) of length
      len(CONTOURS)+1. If you do not specify them, equally
      spaced colors are chosen.

      If CONTOURS is an integer, SCALE expresses how contour levels
      are determined.  SCALE may be "lin", "log", or "normal"
      specifying linearly, logarithmically, or normally spaced
      contours. Note that unlike Yorick's plfc, this routine does
      not use spann to compute its contours. Neither, apparently,
      does plc, which uses a third algorithm which matches neither
      the one we use nor the one spann uses. So if you plot filled
      contours and then plot contour lines, the contours will in
      general not coincide exactly.

      Note that you may use spann to calculate your contour levels
      if you wish.

      The following keywords are legal (each has a separate help entry):
    KEYWORDS: triangle, region
    SEE ALSO: plg, plm, plc, plv, plf, pli, plt, pldj, plfp, plmesh
              color_bar, spann, contour, limits, logxy, range, fma, hcp
   """
   # 1. Get contour colors

   (vcmin, vcmax) = zmin_zmax (z, ireg)
   if type (contours) == IntType :
      n = contours
      vc = zeros (n + 2, Float)
      vc [0] = vcmin
      vc [n + 1] = vcmax
      if scale == "lin" or scale is None :
          #    This stuff is in lieu of the spann stuff in Yorick.
          vc [1:n + 1] = vcmin + arange (1, n + 1, typecode = Float) * \
             (vcmax - vcmin) / (n + 1)
      elif scale == "log" :
          vc [1:n + 1] = vcmin + exp (arange (1, n + 1, typecode = Float) * \
             log (vcmax - vcmin) / (n + 1))
      elif scale == "normal" :
          zlin = ravel (z)
          lzlin = len (zlin)
          zbar = add.reduce (zlin) / lzlin
          zs = sqrt ( (add.reduce (zlin ** 2) - lzlin * zbar ** 2) /
              (lzlin - 1))
          z1 = zbar - 2. * zs
          z2 = zbar + 2. * zs
          diff = (z2 - z1) / (n - 1)
          vc [1:n + 1] = z1 + arange (n) * diff
      else :
          raise _ContourError, "Incomprehensible scale parameter."
   elif type (contours) == ArrayType and contours.typecode () == Float :
      n = len (contours)
      vc = zeros (n + 2, Float)
      vc [0] = vcmin
      vc [n + 1] = vcmax
      vc [1:n + 1] = sort (contours)
   else :
      raise _ContourError, "Incorrect contour specification."
   if colors is None :
      colors = (arange (n + 1, typecode = Float) * (199. / n)).astype ('b')
   else :
      colors = array (colors)
      if len (colors) != n + 1 :
         raise "PLFC_Error", \
            "colors must specify one more color than contours."
      if colors.typecode != 'b' :
         colors = bytscl (colors)

   if triangle is None :
      triangle = zeros (z.shape, Int16)

  # Set mesh first
   plmesh (y, x, ireg, triangle = triangle)
   for i in range (n + 1) :
      [nc, yc, xc] = contour (array ( [vc [i], vc [i + 1]]), z)
      if (is_scalar(nc) and nc == 0 or nc is None) :
         continue
      plfp ( (ones (len (nc)) * colors [i]).astype ('b'),
         yc, xc, nc, edges = 0)

def plh (y, x=None, width=1, hide=0, color=None, labels=None, height=None):
   """
   plh ( y, [x, labels, <keylist>])
      draws a histogram, where the height of the bars is given
      by Y. If X is None, the bars in the histogram have a
      width equal to unity. If X is a single real number, the
      widths of the bars are all equal to this number. If X is
      a one-dimensional array with an equal number of elements
      as the Y array, then X is interpreted as the widths of
      the bars. In all of these cases, the histogram starts at
      the origin. However, if X is a one-dimensional array
      with one element more than Y, then X is interpreted as
      the locations of the start and end points of the bars in
      the histogram. If X is a one-dimensional array with twice
      as many elements as Y, then X represents the start and
      end points for each bar separately.
      The keyword color is either a single color, representing
      the fill color of the bars, or a list of colors, one for
      each bar.
      If the keyword labels is given, then the horizontal tick
      marks and numerical labels are switched off. The keyword
      labels should then consist of a list of strings, with the
      same number of elements as Y. These labels are then drawn
      below the horizontal axis. The keyword height, if given,
      specifies the font height for the labels.
      To switch the tick marks and labels back on for subsequent
      plots, you can execute
      window(style="work.gs")
      which will reset the window to the usual work.gs style sheet.
      
      
      The following keywords are legal (each has a separate help entry):
    KEYWORDS: width, hide, color, height
    SEE ALSO: plg, plm, plc, plv, plf, pli, plt, pldj, plfp, plmesh
              color_bar, spann, contour, limits, logxy, range, fma, hcp
   """

   color_dict = { 'bg':-1, 'fg':-2, 'black':-3, 'white':-4, 'red':-5,
      'green':-6, 'blue':-7, 'cyan':-8, 'magenta':-9, 'yellow':-10 }
   n = len(y)
   barx = [[]] * n
   if x is None:
      for i in range(n):
         barx[i] = array([i,i,i+1,i+1])
   else:
      if type(x) == IntType or type(x) == FloatType:
         # x denotes the width of the bars, which are all equal
         for i in range(n-1):
            barx[i] = array([i,i,i+1,i+1]) * x
      elif type(x) == ListType or type(x) == ArrayType:
         if len(x) == n:
            # x denotes the width of the bars, which can be different
            offset = 0
            for i in range(n):
               barx[i] = offset + array([0,0,1,1]) * x[i]
               offset = offset + x[i]
         elif len(x) == n + 1:
            for i in range(n):
               barx[i] = array([x[i],x[i],x[i+1],x[i+1]])
         elif len(x) == 2*n:
            for i in range(n):
               barx[i] = array([x[2*i],x[2*i],x[2*i+1],x[2*i+1]])
         else:
            raise "plh error: inconsistent length of X"
   bary = [[]] * n
   for i in range(n):
      bary[i] = array([0,y[i],y[i],0])
   if labels:
      if current_window() < 0:
         window(style="boxed.gs",legends=0)
      style = get_style()
      if style['systems'][0]['ticks']['horizontal']['flags'] & 99:
         # We need to switch off tick marks and labels here
         window(style="boxed.gs", legends=0)
         style = get_style()
         flags = style['systems'][0]['ticks']['horizontal']['flags']
         flags = flags & ( ~ 99) # Switch off horizontal tick marks, labels
         style['systems'][0]['ticks']['horizontal']['flags'] = flags
         set_style(style)
   if color:
      if type(color) != ListType:
         color = [color] * n
      for i in range(n):
         z = color[i]
         if type(z) == StringType: z = color_dict[z]
         plfp(array([z],'b'),bary[i],barx[i],[4])
   for i in range(n):
      plg(bary[i],barx[i],width=width,hide=hide,marks=0)
   if labels:
      [left,right,bottom,top] = viewport()
      hticks = style['systems'][0]['ticks']['horizontal']
      scale = (right-left)/(barx[-1][-1]-barx[0][0])
      y = bottom - hticks['labelOff'] + hticks['tickLen'][0] + hticks['tickOff']
      if height:
         for i in range(n):
            x = left + scale * ((barx[i][0]+barx[i][-1])/2. - barx[0][0])
            plt(labels[i],x,y,justify="CT",height=height)
      else:
         for i in range(n):
            x = left + scale * ((barx[i][0]+barx[i][-1])/2. - barx[0][0])
            plt(labels[i],x,y,justify="CT")





import _zeros

_iter = 100
_xtol = 1e-12
_args = ()

__all__ = ['bisect','ridder','brentq','brenth']
    
def bisect(f,a,b,xtol=_xtol,maxiter=_iter,args=_args, full_output=0, disp=1) :
    """ Find root of f in [a,b]

        Basic bisection routine to find a zero of the function
        f between the arguments a and b. f(a) and f(b) can not
        have the same signs. Slow but sure.

        f : Python function returning a number.

        a : Number, one end of the bracketing interval.

        b : Number, the other end of the bracketing interval.

        xtol : Number, the routine converges when a root is known
            to lie within xtol of the value return. Should be >= 0.
            The routine modifies this to take into account the relative
            precision of doubles.
            
        maxiter : Number, if convergence is not achieved in
            maxiter iterations, and error is raised. Must be
            >= 0.

        args : tuple containing extra arguments for the function f.
            f is called by apply(f,(x)+args).
    """
    if type(args) != type(()) :
	    args = (args,)
    return _zeros._bisect(f,a,b,xtol,maxiter,args,full_output,disp)

def ridder(f,a,b,xtol=_xtol,maxiter=_iter,args=_args,full_output=0,disp=1) :
    """ Find root of f in [a,b]

        Ridder routine to find a zero of the function
        f between the arguments a and b. f(a) and f(b) can not
        have the same signs. Faster than bisection, but not
        generaly as fast as the brent rountines. A description
        may be found in a recent edition of Numerical Recipes.
        The routine here is modified a bit to be more careful
        of tolerance.

        f : Python function returning a number.

        a : Number, one end of the bracketing interval.

        b : Number, the other end of the bracketing interval.

        xtol : Number, the routine converges when a root is known
            to lie within xtol of the value return. Should be >= 0.
            The routine modifies this to take into account the relative
            precision of doubles.
            
        maxiter : Number, if convergence is not achieved in
            maxiter iterations, and error is raised. Must be
            >= 0.

        args : tuple containing extra arguments for the function f.
            f is called by apply(f,(x)+args).
    """
    if type(args) != type(()) :
	    args = (args,)
    return _zeros._ridder(f,a,b,xtol,maxiter,args,full_output,disp)

def brentq(f,a,b,xtol=_xtol,maxiter=_iter,args=_args,full_output=0,disp=1) :
    """ Find root of f in [a,b]

        The classic Brent routine to find a zero of the function
        f between the arguments a and b. f(a) and f(b) can not
        have the same signs. Generally the best of the routines here.
        It is a safe version of the secant method that uses inverse
        quadratic extrapolation. The version here is a slight
        modification that uses a different formula in the extrapolation
        step. A description may be found in Numerical Recipes, but the
        code here is probably easier to understand.

        f : Python function returning a number.

        a : Number, one end of the bracketing interval.

        b : Number, the other end of the bracketing interval.

        xtol : Number, the routine converges when a root is known
            to lie within xtol of the value return. Should be >= 0.
            The routine modifies this to take into account the relative
            precision of doubles.
            
        maxiter : Number, if convergence is not achieved in
            maxiter iterations, and error is raised. Must be
            >= 0.

        args : tuple containing extra arguments for the function f.
            f is called by apply(f,(x)+args).
    """
    if type(args) != type(()) :
	    args = (args,)
    return _zeros._brentq(f,a,b,xtol,maxiter,args,full_output,disp)

def brenth(f,a,b,xtol=_xtol,maxiter=_iter,args=_args,full_output=0,disp=1) :
    """ Find root of f in [a,b]

        A variation on the classic Brent routine to find a zero
        of the function f between the arguments a and b that uses
        hyperbolic extrapolation instead of inverse quadratic
        extrapolation. There was a paper back in the 1980's ...
        f(a) and f(b) can not have the same signs. Generally on a
        par with the brent routine, but not as heavily tested.
        It is a safe version of the secant method that uses hyperbolic
        extrapolation. The version here is by Chuck Harris.

        f : Python function returning a number.

        a : Number, one end of the bracketing interval.

        b : Number, the other end of the bracketing interval.

        xtol : Number, the routine converges when a root is known
            to lie within xtol of the value return. Should be >= 0.
            The routine modifies this to take into account the relative
            precision of doubles.
            
        maxiter : Number, if convergence is not achieved in
            maxiter iterations, and error is raised. Must be
            >= 0.

        args : tuple containing extra arguments for the function f.
            f is called by apply(f,(x)+args).
    """
    if type(args) != type(()) :
	    args = (args,)
    return _zeros._brenth(f,a,b,xtol,maxiter,args,full_output,disp)




import os
import re
import sys

from fcompiler import FCompiler
import log

class IbmFCompiler(FCompiler):

    compiler_type = 'ibm'
    version_pattern =  r'xlf\(1\)\s*IBM XL Fortran (Advanced Edition |)Version (?P<version>[^\s*]*)'

    executables = {
        'version_cmd'  : ["xlf"],
        'compiler_f77' : ["xlf"],
        'compiler_fix' : ["xlf90", "-qfixed"],
        'compiler_f90' : ["xlf90"],
        'linker_so'    : ["xlf95"],
        'archiver'     : ["ar", "-cr"],
        'ranlib'       : ["ranlib"]
        }

    def get_version(self,*args,**kwds):
        version = FCompiler.get_version(self,*args,**kwds)
        xlf_dir = '/etc/opt/ibmcmp/xlf'
        if version is None and os.path.isdir(xlf_dir):
            # If the output of xlf does not contain version info
            # (that's the case with xlf 8.1, for instance) then
            # let's try another method:
            l = os.listdir(xlf_dir)
            l.sort()
            l.reverse()
            l = [d for d in l if os.path.isfile(os.path.join(xlf_dir,d,'xlf.cfg'))]
            if not l:
                from distutils.version import LooseVersion
                self.version = version = LooseVersion(l[0])
        return version

    def get_flags(self):
        return ['-qextname']

    def get_flags_debug(self):
        return ['-g']

    def get_flags_linker_so(self):
        opt = []
        if sys.platform=='darwin':
            opt.append('-Wl,-bundle,-flat_namespace,-undefined,suppress')
        else:
            opt.append('-bshared')
        version = self.get_version(ok_status=[0,40])
        if version is not None:
            import tempfile
            xlf_cfg = '/etc/opt/ibmcmp/xlf/%s/xlf.cfg' % version
            new_cfg = tempfile.mktemp()+'_xlf.cfg'
            log.info('Creating '+new_cfg)
            fi = open(xlf_cfg,'r')
            fo = open(new_cfg,'w')
            crt1_match = re.compile(r'\s*crt\s*[=]\s*(?P<path>.*)/crt1.o').match
            for line in fi.readlines():
                m = crt1_match(line)
                if m:
                    fo.write('crt = %s/bundle1.o\n' % (m.group('path')))
                else:
                    fo.write(line)
            fi.close()
            fo.close()
            opt.append('-F'+new_cfg)
        return opt

    def get_flags_opt(self):
        return ['-O5']

if __name__ == '__main__':
    from distutils import log
    log.set_verbosity(2)
    from fcompiler import new_fcompiler
    compiler = new_fcompiler(compiler='ibm')
    compiler.customize()
    print compiler.get_version()



"""
Integration routines
====================

 Methods for Integrating Functions given function object.

   quad          -- General purpose integration.
   dblquad       -- General purpose double integration.
   tplquad       -- General purpose triple integration.
   fixed_quad    -- Integrate func(x) using Gaussian quadrature of order n.
   quadrature    -- Integrate with given tolerance using Gaussian quadrature.
   romberg       -- Integrate func using Romberg integration.

 Methods for Integrating Functions given fixed samples.

   trapz         -- Use trapezoidal rule to compute integral from samples.
   cumtrapz      -- Use trapezoidal rule to cumulatively compute integral.
   simps         -- Use Simpson's rule to compute integral from samples.
   romb          -- Use Romberg Integration to compute integral from
                    (2**k + 1) evenly-spaced samples.

   See the special module's orthogonal polynomials (special) for Gaussian
      quadrature roots and weights for other weighting factors and regions.

 Interface to numerical integrators of ODE systems.

   odeint        -- General integration of ordinary differential equations.
   ode           -- Integrate ODE using vode routine.

"""

postpone_import = 1




__all__ = []

import os,sys,time,glob,string,traceback,unittest
import types
import imp

try:
    # These are used by Numeric tests.
    # If Numeric and scipy_base  are not available, then some of the
    # functions below will not be available.
    from Numeric import alltrue,equal,shape,ravel,around,zeros,Float64,asarray,\
         less_equal,array2string,less
    import scipy_base.fastumath as math
except ImportError:
    pass

DEBUG=0

__all__.append('set_package_path')
def set_package_path(level=1):
    """ Prepend package directory to sys.path.

    set_package_path should be called from a test_file.py that
    satisfies the following tree structure:

      <somepath>/<somedir>/test_file.py

    Then the first existing path name from the following list

      <somepath>/build/lib.<platform>-<version>
      <somepath>/..

    is prepended to sys.path.
    The caller is responsible for removing this path by using

      restore_path()
    """
    from distutils.util import get_platform
    from scipy_distutils.misc_util import get_frame
    f = get_frame(level)
    if f.f_locals['__name__']=='__main__':
        testfile = sys.argv[0]
    else:
        testfile = f.f_locals['__file__']
    d = os.path.dirname(os.path.dirname(os.path.abspath(testfile)))
    d1 = os.path.join(d,'build','lib.%s-%s'%(get_platform(),sys.version[:3]))
    if not os.path.isdir(d1):
        d1 = os.path.dirname(d)
    if DEBUG:
        print 'Inserting %r to sys.path' % (d1)
    sys.path.insert(0,d1)

__all__.append('set_local_path')
def set_local_path(reldir='', level=1):
    """ Prepend local directory to sys.path.

    The caller is responsible for removing this path by using

      restore_path()
    """
    from scipy_distutils.misc_util import get_frame
    f = get_frame(level)
    if f.f_locals['__name__']=='__main__':
        testfile = sys.argv[0]
    else:
        testfile = f.f_locals['__file__']
    local_path = os.path.join(os.path.dirname(os.path.abspath(testfile)),reldir)
    if DEBUG:
        print 'Inserting %r to sys.path' % (local_path)
    sys.path.insert(0,local_path)

__all__.append('restore_path')
def restore_path():
    if DEBUG:
        print 'Removing %r from sys.path' % (sys.path[0])
    del sys.path[0]

if sys.platform[:5]=='linux':
    def jiffies(_proc_pid_stat = '/proc/%s/stat'%(os.getpid()),
                _load_time=time.time()):
        """ Return number of jiffies (1/100ths of a second) that this
    process has been scheduled in user mode. See man 5 proc. """
        try:
            f=open(_proc_pid_stat,'r')
            l = f.readline().split(' ')
            f.close()
            return int(l[13])
        except:
            return int(100*(time.time()-_load_time))

    def memusage(_proc_pid_stat = '/proc/%s/stat'%(os.getpid())):
        """ Return virtual memory size in bytes of the running python.
        """
        try:
            f=open(_proc_pid_stat,'r')
            l = f.readline().split(' ')
            f.close()
            return int(l[22])
        except:
            return
else:
    # os.getpid is not in all platforms available.
    # Using time is safe but inaccurate, especially when process
    # was suspended or sleeping.
    def jiffies(_load_time=time.time()):
        """ Return number of jiffies (1/100ths of a second) that this
    process has been scheduled in user mode. [Emulation with time.time]. """
        return int(100*(time.time()-_load_time))

    def memusage():
        """ Return memory usage of running python. [Not implemented]"""
        return

__all__.append('ScipyTestCase')
class ScipyTestCase (unittest.TestCase):

    def measure(self,code_str,times=1):
        """ Return elapsed time for executing code_str in the
        namespace of the caller for given times.
        """
        frame = sys._getframe(1)
        locs,globs = frame.f_locals,frame.f_globals
        code = compile(code_str,
                       'ScipyTestCase runner for '+self.__class__.__name__,
                       'exec')
        i = 0
        elapsed = jiffies()
        while i<times:
            i += 1
            exec code in globs,locs
        elapsed = jiffies() - elapsed
        return 0.01*elapsed

    def __call__(self, result=None):
        if result is None:
            return unittest.TestCase.__call__(self, result)

        nof_errors = len(result.errors)
        save_stream = result.stream
        result.stream = _dummy_stream()
        unittest.TestCase.__call__(self, result)
        if nof_errors != len(result.errors):
            test, errstr = result.errors[-1]
            if errstr.split('\n')[-2].startswith('IgnoreException:'):
                assert result.stream.data[-1]=='E',`result.stream.data`
                result.stream.data[-1] = 'i'
                del result.errors[-1]
        map(save_stream.write, result.stream.data)
        result.stream = save_stream

class _dummy_stream:
    def __init__(self):
        self.data = []
    def write(self,message):
        self.data.append(message)
    def writeln(self,message):
        self.data.append(message+'\n')

__all__.append('IgnoreException')
class IgnoreException(Exception):
    "Ignoring this exception due to disabled feature"

#------------

def _get_all_method_names(cls):
    names = dir(cls)
    if sys.version[:3]<='2.1':
        for b in cls.__bases__:
            for n in dir(b)+_get_all_method_names(b):
                if n not in names:
                    names.append(n)
    return names

__all__.append('ScipyTest')
class ScipyTest:
    """ Scipy tests site manager.

    Usage:
      >>> ScipyTest(<package>).test(level=1,verbosity=2)

    <package> is package name or its module object.

    Package is supposed to contain a directory tests/
    with test_*.py files where * refers to the names of submodules.

    test_*.py files are supposed to define a classes, derived
    from ScipyTestCase or unittest.TestCase, with methods having
    names starting with test or bench or check.

    And that is it! No need to implement test or test_suite functions
    in each .py file.

    Also old styled test_suite(level=1) hooks are supported but
    soon to be removed.
    """
    def __init__(self, package='__main__'):
        self.package = package

    def _module_str(self, module):
        filename = module.__file__[-30:]
        if filename!=module.__file__:
            filename = '...'+filename
        return '<module %s from %s>' % (`module.__name__`, `filename`)

    def _get_method_names(self,clsobj,level):
        names = []
        for mthname in _get_all_method_names(clsobj):
            if mthname[:5] not in ['bench','check'] \
               and mthname[:4] not in ['test']:
                continue
            mth = getattr(clsobj, mthname)
            if type(mth) is not types.MethodType:
                continue
            d = mth.im_func.func_defaults
            if d is not None:
                mthlevel = d[0]
            else:
                mthlevel = 1
            if level>=mthlevel:
                if mthname not in names:
                    names.append(mthname)
            for base in clsobj.__bases__:
                for n in self._get_method_names(base,level):
                    if n not in names:
                        names.append(n)
        return names

    def _get_module_tests(self,module,level):
        mstr = self._module_str
        d,f = os.path.split(module.__file__)

        short_module_name = os.path.splitext(os.path.basename(f))[0]
        test_dir = os.path.join(d,'tests')
        test_file = os.path.join(test_dir,'test_'+short_module_name+'.py')

        local_test_dir = os.path.join(os.getcwd(),'tests')
        local_test_file = os.path.join(local_test_dir,
                                       'test_'+short_module_name+'.py')
        if os.path.basename(os.path.dirname(local_test_dir)) \
               == os.path.basename(os.path.dirname(test_dir)) \
           and os.path.isfile(local_test_file):
            test_file = local_test_file

        if not os.path.isfile(test_file):
            print '   !! No test file %r found for %s' \
                  % (os.path.basename(test_file), mstr(module))
            return []

        try:
            if sys.version[:3]=='2.1':
                # Workaround for Python 2.1 .pyc file generator bug
                import random
                pref = '-nopyc'+`random.randint(1,100)`
            else:
                pref = ''
            f = open(test_file,'r')
            test_module = imp.load_module(\
                module.__name__+'.test_'+short_module_name+pref,
                f, test_file+pref,('.py', 'r', 1))
            f.close()
            if sys.version[:3]=='2.1' and os.path.isfile(test_file+pref+'c'):
                os.remove(test_file+pref+'c')
        except:
            print '   !! FAILURE importing tests for ', mstr(module)
            print '   ',
            output_exception()
            return []
        return self._get_suite_list(test_module, level, module.__name__)

    def _get_suite_list(self, test_module, level, module_name='__main__'):
        mstr = self._module_str
        if hasattr(test_module,'test_suite'):
            # Using old styled test suite
            try:
                total_suite = test_module.test_suite(level)
                return total_suite._tests
            except:
                print '   !! FAILURE building tests for ', mstr(test_module)
                print '   ',
                output_exception()
                return []
        suite_list = []
        for name in dir(test_module):
            obj = getattr(test_module, name)
            if type(obj) is not type(unittest.TestCase) \
               or not issubclass(obj, unittest.TestCase) \
               or obj.__name__[:4] != 'test':
                continue
            suite_list.extend(map(obj,self._get_method_names(obj,level)))
        print '  Found',len(suite_list),'tests for',module_name
        return suite_list

    def _touch_ppimported(self, module):
        from scipy_base.ppimport import _ModuleLoader
        if os.path.isdir(os.path.join(os.path.dirname(module.__file__),'tests')):
            # only touching those modules that have tests/ directory
            try: module._pliuh_plauh
            except AttributeError: pass
            for name in dir(module):
                obj = getattr(module,name)
                if isinstance(obj,_ModuleLoader) \
                   and not hasattr(obj,'_ppimport_module') \
                   and not hasattr(obj,'_ppimport_exc_info'):
                    self._touch_ppimported(obj)

    def test(self,level=1,verbosity=1):
        """ Run Scipy module test suite with level and verbosity.
        """
        if type(self.package) is type(''):
            exec 'import %s as this_package' % (self.package)
        else:
            this_package = self.package

        self._touch_ppimported(this_package)

        package_name = this_package.__name__

        suites = []
        for name, module in sys.modules.items():
            if package_name != name[:len(package_name)] \
                   or module is None \
                   or os.path.basename(os.path.dirname(module.__file__))=='tests':
                continue
            suites.extend(self._get_module_tests(module, level))

        suites.extend(self._get_suite_list(sys.modules[package_name], level))

        all_tests = unittest.TestSuite(suites)
        runner = unittest.TextTestRunner(verbosity=verbosity)
        runner.run(all_tests)
        return runner

    def run(self):
        """ Run Scipy module test suite with level and verbosity
        taken from sys.argv. Requires optparse module.
        """
        try:
            from optparse import OptionParser
        except ImportError:
            print 'Failed to import optparse module, ignoring.'
            return self.test()
        usage = r'usage: %prog [<options>]'
        parser = OptionParser(usage)
        parser.add_option("-v", "--verbosity",
                          action="store",
                          dest="verbosity",
                          default=1,
                          type='int')
        parser.add_option("-l", "--level",
                          action="store",
                          dest="level",
                          default=1,
                          type='int')
        (options, args) = parser.parse_args()
        self.test(options.level,options.verbosity)

#------------
        
def remove_ignored_patterns(files,pattern):
    from fnmatch import fnmatch
    good_files = []
    for file in files:
        if not fnmatch(file,pattern):
            good_files.append(file)
    return good_files

def remove_ignored_files(original,ignored_files,cur_dir):
    """ This is actually expanded to do pattern matching.

    """
    if not ignored_files: ignored_files = []
    ignored_modules = map(lambda x: x+'.py',ignored_files)
    ignored_packages = ignored_files[:]
    # always ignore setup.py and __init__.py files
    ignored_files = ['setup.py','setup_*.py','__init__.py']
    ignored_files += ignored_modules + ignored_packages
    ignored_files = map(lambda x,cur_dir=cur_dir: os.path.join(cur_dir,x),
                        ignored_files)
    #print 'ignored:', ignored_files
    #good_files = filter(lambda x,ignored = ignored_files: x not in ignored,
    #                    original)
    good_files = original
    for pattern in ignored_files:
        good_files = remove_ignored_patterns(good_files,pattern)

    return good_files

__all__.append('harvest_modules')
def harvest_modules(package,ignore=None):
    """* Retreive a list of all modules that live within a package.

         Only retreive files that are immediate children of the
         package -- do not recurse through child packages or
         directories.  The returned list contains actual modules, not
         just their names.
    *"""
    d,f = os.path.split(package.__file__)

    # go through the directory and import every py file there.
    common_dir = os.path.join(d,'*.py')
    py_files = glob.glob(common_dir)
    #py_files.remove(os.path.join(d,'__init__.py'))
    #py_files.remove(os.path.join(d,'setup.py'))

    py_files = remove_ignored_files(py_files,ignore,d)
    #print 'py_files:', py_files
    try:
        prefix = package.__name__
    except:
        prefix = ''

    all_modules = []
    for file in py_files:
        d,f = os.path.split(file)
        base,ext =  os.path.splitext(f)
        mod = prefix + '.' + base
        #print 'module: import ' + mod
        try:
            exec ('import ' + mod)
            all_modules.append(eval(mod))
        except:
            print 'FAILURE to import ' + mod
            output_exception()

    return all_modules

__all__.append('harvest_packages')
def harvest_packages(package,ignore = None):
    """ Retreive a list of all sub-packages that live within a package.

         Only retreive packages that are immediate children of this
         package -- do not recurse through child packages or
         directories.  The returned list contains actual package objects, not
         just their names.
    """
    join = os.path.join

    d,f = os.path.split(package.__file__)

    common_dir = os.path.abspath(d)
    all_files = os.listdir(d)

    all_files = remove_ignored_files(all_files,ignore,'')
    #print 'all_files:', all_files
    try:
        prefix = package.__name__
    except:
        prefix = ''
    all_packages = []
    for directory in all_files:
        path = join(common_dir,directory)
        if os.path.isdir(path) and \
           os.path.exists(join(path,'__init__.py')):
            sub_package = prefix + '.' + directory
            #print 'sub-package import ' + sub_package
            try:
                exec ('import ' + sub_package)
                all_packages.append(eval(sub_package))
            except:
                print 'FAILURE to import ' + sub_package
                output_exception()
    return all_packages

__all__.append('harvest_modules_and_packages')
def harvest_modules_and_packages(package,ignore=None):
    """ Retreive list of all packages and modules that live within a package.

         See harvest_packages() and harvest_modules()
    """
    all = harvest_modules(package,ignore) + harvest_packages(package,ignore)
    return all

__all__.append('harvest_test_suites')
def harvest_test_suites(package,ignore = None,level=10):
    """
        package -- the module to test.  This is an actual module object
                   (not a string)
        ignore  -- a list of module names to omit from the tests
        level   -- a value between 1 and 10.  1 will run the minimum number
                   of tests.  This is a fast "smoke test".  Tests that take
                   longer to run should have higher numbers ranging up to 10.
    """
    suites=[]
    test_modules = harvest_modules_and_packages(package,ignore)
    #for i in test_modules:
    #    print i.__name__
    for module in test_modules:
        if hasattr(module,'test_suite'):
            try:
                suite = module.test_suite(level=level)
                if suite:
                    suites.append(suite)
                else:
                    print "    !! FAILURE without error - shouldn't happen",
                    print module.__name__
            except:
                print '   !! FAILURE building test for ', module.__name__
                print '   ',
                output_exception()
        else:
            try:
                print 'No test suite found for ', module.__name__
            except AttributeError:
                # __version__.py getting replaced by a string throws a kink
                # in checking for modules, so we think is a module has
                # actually been overwritten
                print 'No test suite found for ', str(module)
    total_suite = unittest.TestSuite(suites)
    return total_suite

__all__.append('module_test')
def module_test(mod_name,mod_file,level=10):
    """*

    *"""
    #print 'testing', mod_name
    d,f = os.path.split(mod_file)

    # insert the tests directory to the python path
    test_dir = os.path.join(d,'tests')
    sys.path.insert(0,test_dir)

    # call the "test_xxx.test()" function for the appropriate
    # module.

    # This should deal with package naming issues correctly
    short_mod_name = string.split(mod_name,'.')[-1]
    test_module = 'test_' + short_mod_name
    test_string = 'import %s;reload(%s);%s.test(%d)' % \
                  ((test_module,)*3 + (level,))

    # This would be better cause it forces a reload of the orginal
    # module.  It doesn't behave with packages however.
    #test_string = 'reload(%s);import %s;reload(%s);%s.test(%d)' % \
    #              ((mod_name,) + (test_module,)*3)
    exec(test_string)

    # remove test directory from python path.
    sys.path = sys.path[1:]

__all__.append('module_test_suite')
def module_test_suite(mod_name,mod_file,level=10):
    #try:
        print ' creating test suite for:', mod_name
        d,f = os.path.split(mod_file)

        # insert the tests directory to the python path
        test_dir = os.path.join(d,'tests')
        sys.path.insert(0,test_dir)

        # call the "test_xxx.test()" function for the appropriate
        # module.

        # This should deal with package naming issues correctly
        short_mod_name = string.split(mod_name,'.')[-1]
        test_module = 'test_' + short_mod_name
        test_string = 'import %s;reload(%s);suite = %s.test_suite(%d)' % \
                      ((test_module,)*3+(level,))
        #print test_string
        exec(test_string)

        # remove test directory from python path.
        sys.path = sys.path[1:]
        return suite
    #except:
    #    print '    !! FAILURE loading test suite from', test_module, ':'
    #    print '   ',
    #    output_exception()


# Utility function to facilitate testing.

__all__.append('assert_equal')
def assert_equal(actual,desired,err_msg='',verbose=1):
    """ Raise an assertion if two items are not
        equal.  I think this should be part of unittest.py
    """
    msg = '\nItems are not equal:\n' + err_msg
    try:
        if ( verbose and len(repr(desired)) < 100 and len(repr(actual)) ):
            msg =  msg \
                 + 'DESIRED: ' + repr(desired) \
                 + '\nACTUAL: ' + repr(actual)
    except:
        msg =  msg \
             + 'DESIRED: ' + repr(desired) \
             + '\nACTUAL: ' + repr(actual)
    assert desired == actual, msg

__all__.append('assert_almost_equal')
def assert_almost_equal(actual,desired,decimal=7,err_msg='',verbose=1):
    """ Raise an assertion if two items are not
        equal.  I think this should be part of unittest.py
    """
    msg = '\nItems are not equal:\n' + err_msg
    try:
        if ( verbose and len(repr(desired)) < 100 and len(repr(actual)) ):
            msg =  msg \
                 + 'DESIRED: ' + repr(desired) \
                 + '\nACTUAL: ' + repr(actual)
    except:
        msg =  msg \
             + 'DESIRED: ' + repr(desired) \
             + '\nACTUAL: ' + repr(actual)
    assert round(abs(desired - actual),decimal) == 0, msg

__all__.append('assert_approx_equal')
def assert_approx_equal(actual,desired,significant=7,err_msg='',verbose=1):
    """ Raise an assertion if two items are not
        equal.  I think this should be part of unittest.py
        Approximately equal is defined as the number of significant digits
        correct
    """
    msg = '\nItems are not equal to %d significant digits:\n' % significant
    msg += err_msg
    actual, desired = map(float, (actual, desired))
    # Normalized the numbers to be in range (-10.0,10.0)
    scale = pow(10,math.floor(math.log10(0.5*(abs(desired)+abs(actual)))))
    try:
        sc_desired = desired/scale
    except ZeroDivisionError:
        sc_desired = 0.0
    try:
        sc_actual = actual/scale
    except ZeroDivisionError:
        sc_actual = 0.0
    try:
        if ( verbose and len(repr(desired)) < 100 and len(repr(actual)) ):
            msg =  msg \
                 + 'DESIRED: ' + repr(desired) \
                 + '\nACTUAL: ' + repr(actual)
    except:
        msg =  msg \
             + 'DESIRED: ' + repr(desired) \
             + '\nACTUAL: ' + repr(actual)
    assert math.fabs(sc_desired - sc_actual) < pow(10.,-1*significant), msg


__all__.append('assert_array_equal')
def assert_array_equal(x,y,err_msg=''):
    x,y = asarray(x), asarray(y)
    msg = '\nArrays are not equal'
    try:
        assert 0 in [len(shape(x)),len(shape(y))] \
               or (len(shape(x))==len(shape(y)) and \
                   alltrue(equal(shape(x),shape(y)))),\
                   msg + ' (shapes %s, %s mismatch):\n\t' \
                   % (shape(x),shape(y)) + err_msg
        reduced = ravel(equal(x,y))
        cond = alltrue(reduced)
        if not cond:
            s1 = array2string(x,precision=16)
            s2 = array2string(y,precision=16)
            if len(s1)>120: s1 = s1[:120] + '...'
            if len(s2)>120: s2 = s2[:120] + '...'
            match = 100-100.0*reduced.tolist().count(1)/len(reduced)
            msg = msg + ' (mismatch %s%%):\n\tArray 1: %s\n\tArray 2: %s' % (match,s1,s2)
        assert cond,\
               msg + '\n\t' + err_msg
    except ValueError:
        raise ValueError, msg

__all__.append('assert_array_almost_equal')
def assert_array_almost_equal(x,y,decimal=6,err_msg=''):
    x = asarray(x)
    y = asarray(y)
    msg = '\nArrays are not almost equal'
    try:
        cond = alltrue(equal(shape(x),shape(y)))
        if not cond:
            msg = msg + ' (shapes mismatch):\n\t'\
                  'Shape of array 1: %s\n\tShape of array 2: %s' % (shape(x),shape(y))
        assert cond, msg + '\n\t' + err_msg
        reduced = ravel(equal(less_equal(around(abs(x-y),decimal),10.0**(-decimal)),1))
        cond = alltrue(reduced)
        if not cond:
            s1 = array2string(x,precision=decimal+1)
            s2 = array2string(y,precision=decimal+1)
            if len(s1)>120: s1 = s1[:120] + '...'
            if len(s2)>120: s2 = s2[:120] + '...'
            match = 100-100.0*reduced.tolist().count(1)/len(reduced)
            msg = msg + ' (mismatch %s%%):\n\tArray 1: %s\n\tArray 2: %s' % (match,s1,s2)
        assert cond,\
               msg + '\n\t' + err_msg
    except ValueError:
        print sys.exc_value
        print shape(x),shape(y)
        print x, y
        raise ValueError, 'arrays are not almost equal'

__all__.append('assert_array_less')
def assert_array_less(x,y,err_msg=''):
    x,y = asarray(x), asarray(y)
    msg = '\nArrays are not less-ordered'
    try:
        assert alltrue(equal(shape(x),shape(y))),\
               msg + ' (shapes mismatch):\n\t' + err_msg
        reduced = ravel(less(x,y))
        cond = alltrue(reduced)
        if not cond:
            s1 = array2string(x,precision=16)
            s2 = array2string(y,precision=16)
            if len(s1)>120: s1 = s1[:120] + '...'
            if len(s2)>120: s2 = s2[:120] + '...'
            match = 100-100.0*reduced.tolist().count(1)/len(reduced)
            msg = msg + ' (mismatch %s%%):\n\tArray 1: %s\n\tArray 2: %s' % (match,s1,s2)
        assert cond,\
               msg + '\n\t' + err_msg
    except ValueError:
        print shape(x),shape(y)
        raise ValueError, 'arrays are not less-ordered'

__all__.append('rand')
def rand(*args):
    """ Returns an array of random numbers with the given shape.
    used for testing
    """
    import random
    results = zeros(args,Float64)
    f = results.flat
    for i in range(len(f)):
        f[i] = random.random()
    return results

def output_exception():
    try:
        type, value, tb = sys.exc_info()
        info = traceback.extract_tb(tb)
        #this is more verbose
        #traceback.print_exc()
        filename, lineno, function, text = info[-1] # last line only
        print "%s:%d: %s: %s (in %s)" %\
              (filename, lineno, type.__name__, str(value), function)
    finally:
        type = value = tb = None # clean up



from Numeric import *
from scipy_base.fastumath import *

import sys, thread, sync



import remote_exec	

import population



"""

######

#I've got to lean up evaluate and initial in population so that 

#the incorporation of the parallel stuff is smoother.

######

"""

import sys, thread, sync



def array_round(x):

	y = zeros(shape(x))

	for i in range(len(x.flat)): 

		y[i] = int(round(x[i]))

	return y



def divide_list(l,sections):

		Ntot = len(l)

		Nsec = float(sections)

		Neach = Ntot/Nsec

		div_points = array_round(arange(0,Ntot,Neach)).tolist()		

		if div_points[-1] != Ntot: div_points.append(Ntot)

		sub_pops = []

		st = div_points[0]

		for end in div_points[1:]:

			sub_pops.append(l[st:end])

			st = end

		return sub_pops	



class parallel_pop_initializer:

	def evaluate(self,pop,settings = None):

		#only send the individuals out that need evaluation

		if len(pop):

			Nserv = len(pop.server_list)

			groups = divide_list(pop,Nserv)

			sys.setcheckinterval(10)

			finished = sync.event()

			bar = sync.barrier(Nserv)

			print '************',len(groups), len(pop.server_list), len(pop)

			for i in range(len(groups)):

				inputs = {'sub_pop':groups[i],'settings':settings, 'initializer':pop.initializer}

				returns = ('sub_pop',)

				code = 'initializer.evaluate(sub_pop,settings)'

				data_pack = (inputs,returns,code)

				server = pop.server_list[i]

				thread.start_new_thread(remote_thread_init,(bar,finished,server,data_pack))

			finished.wait()

			sys.setcheckinterval(10)

#what is this?			for ind in pop: ind.evaluate(force)



import cPickle

def plen(obj): return len(cPickle.dumps(obj,1))



class parallel_pop_evaluator:

	def evaluate(self,pop,force = 0):

		import tree

		#print '1',tree.ref()

		#only send the individuals out that need evaluation

		if force:	

			_eval_list = pop.data

		else:	

			_eval_list = filter(lambda x: not x.evaluated,pop)

		#print '2',tree.ref()	

		eval_list = pop.clone()

		#print '3',tree.ref()

		eval_list.data = _eval_list

		if len(eval_list):

			Nserv = len(pop.server_list)

			groups = divide_list(eval_list,Nserv)

			#print '4',tree.ref()

			sys.setcheckinterval(10)

			finished = sync.event()

			bar = sync.barrier(Nserv)

			#print "EVAL LENGTH!!!", plen(pop.evaluator)

			gr = groups[0]

			print "GROUP LENGTH!!!", plen(groups[0]), len(gr), 

			#print "IND!!!", plen(gr[0]),plen(gr[0].root)

			#print '4.5',tree.ref()

			for i in range(len(groups)):

				inputs = {'sub_pop':groups[i], 'evaluator':pop.evaluator, 'force':force}

				returns = ('sub_pop',)

				code = 'evaluator.evaluate(sub_pop,force)'

				data_pack = (inputs,returns,code)

				server = pop.server_list[i]

				thread.start_new_thread(remote_thread_eval,(bar,finished,server,data_pack))

			#print '7',tree.ref()	

			finished.wait()

			sys.setcheckinterval(10)

#what is this?			for ind in pop: ind.evaluate(force)

	"""

	def evaluate(self,pop,force = 0):

		#only send the individuals out that need evaluation

		_eval_list = filter(lambda x: not x.evaluated,pop)

		eval_list = pop.clone()

		eval_list.data = _eval_list

		if len(eval_list):

			#finest grain possible

			groups = divide_list(eval_list,len(eval_list))

			finished = sync.event()

			bar = sync.barrier(groups)

			

			sys.setcheckinterval(10)

			Nserv = len(pop.server_list)

			idx = 0

			while idx < len(groups):

				inputs = {'sub_pop':groups[idx], 'evaluator':pop.evaluator}

				returns = ('sub_pop',)

				code = 'evaluator.evaluate(sub_pop)'

				data_pack = (inputs,returns,code)

				server = pop.server_list[i]

				thread.start_new_thread(remote_thread_eval,(bar,finished,server,data_pack))

			#for i in range(len(groups)):

			#	inputs = {'sub_pop':groups[i], 'evaluator':pop.evaluator}

			#	returns = ('sub_pop',)

			#	code = 'evaluator.evaluate(sub_pop)'

			#	data_pack = (inputs,returns,code)

			#	server = pop.server_list[i]

			#	thread.start_new_thread(remote_thread,(bar,finished,server,data_pack))

			finished.wait()

			sys.setcheckinterval(10)

#what is this?			for ind in pop: ind.evaluate(force)

	"""



def remote_thread_init(bar,finished,server,data_pack):

	try:

		remote = remote_exec.remote_exec(server[0],server[1],0,1)

		results = remote.run(data_pack)

		#assign the results from the returned data to the local individuals

		inputs = data_pack[0]

		old = inputs['sub_pop']

		new = results['sub_pop']

		for i in range(len(old)):

			old[i].__dict__.update(new[i].__dict__)

	except IndexError:

		print 'error in %s,%d' %  server

	bar.enter()

	finished.post()



def remote_thread_eval(bar,finished,server,data_pack):

	import tree

	try:

		#print '5',tree.ref()

		remote = remote_exec.remote_exec(server[0],server[1],0,1)

		results = remote.run(data_pack)

		#print '6',tree.ref()

		#assign the results from the returned data to the local individuals

		inputs = data_pack[0]

		old = inputs['sub_pop']

		new = results['sub_pop']

		for gnm in new:

			gnm.root.delete_circulars()

			del gnm.root

		#print '6.25',tree.ref()

		for i in range(len(old)):

			old[i].__dict__.update(new[i].__dict__)



		#print '6.5',tree.ref()

	except IndexError:

		print 'error in %s,%d' %  server

	"""

	import sys

	#r = new[0].root

	#print 'ref count',sys.getrefcount(r)

	#print '6.75',tree.ref()		

	#Huh??? Why do I need to delete the new genomes

	#individually here?  Why aren't they garbage collected?

	indices = range(len(new))

	indices.reverse()

	for i in indices:

		del new[i]

	#print 'ref count',sys.getrefcount(r)

	#print '6.8',tree.ref()	

	#r.delete_circulars()	

	#print 'ref count',sys.getrefcount(r)

	#print '6.9',tree.ref()	

	#del r

	#print '6.95',tree.ref()	

	"""

	bar.enter()

	finished.post()



class ga_parallel_pop(population.population):

	parallel_evaluator = parallel_pop_evaluator()

	parallel_initializer = parallel_pop_initializer()

	def __init__(self,genome,size=1,server_list=None):

		"""Arguments:

		

		   genome -- a genome object.

		   size -- number.  The population size.  The genome will be 

		           replicated size times to fill the population.

		   server_list -- a list of tuple pairs with machine names and

		                  ports listed for the available servers

		                  ex: [(ee.duke.edu,8000),('elsie.ee.duke.edu',8000)]        

		"""

		population.population.__init__(self,genome,size)

		assert(server_list)

		self.server_list = server_list

	def initialize(self,settings = None):

		"""This method **must** be called before a genetic algorithm 

		   begins evolving the population.  It takes care of initializing

		   the individual genomes, evaluating them, and scaling the population.

		   It also clears and intializes the statistics for the population.

		   

		   Arguments:

		

		   settings -- dictionary of genetic algorithm parameters.  These

		               are passed on to the genomes for initialization.

		"""	

		self.stats = {'current':{},'initial':{},'overall':{}}

		self.stats['ind_evals'] = 0



		print "beigninning genome generation" 

		b = time.clock()	

		self.parallel_initializer.evaluate(self,settings)

		e = time.clock()	

		print "finished generation: ", e-b	

		self.touch(); 

		b = time.clock()	

		self.evaluate()

		e = time.clock()	

		print "evaluation time: ", e-b	

		self.scale()

		self.update_stats()

		self.stats['initial']['avg'] = self.stats['current']['avg']

		self.stats['initial']['max'] = self.stats['current']['max']

		self.stats['initial']['min'] = self.stats['current']['min']

		self.stats['initial']['dev'] = self.stats['current']['dev']

	

	def evaluate(self, force = 0):

		""" call the parallel_evaluator instead of the evaluator directly

		"""

		self.selector.clear()

		self.parallel_evaluator.evaluate(self,force)

		#self.post_evaluate()

		#all of the remaining should be put in post eval...

		self.sort()

		#this is a cluge to get eval count to work correctly

		preval = self.stats['ind_evals']

		for ind in self:  

			self.stats['ind_evals'] = self.stats['ind_evals'] + ind.evals

			ind.evals = 0		

		print 'evals: ', self.stats['ind_evals'] - preval

		self.touch()

		self.evaluated = 1



########################## test stuff ############################

import genome

import gene

import time



import socket			



class objective:

	def __init__(self,wait=.01):

		self.wait = wait

	def evaluate(self,genome): 

		time.sleep(self.wait)

		return sum(genome.array())



def test_pop(server_list,size=100,wait=.01):

	obj = objective(wait)

	the_gene = gene.float_gene((0,2.5))

	genome = genome.list_genome(the_gene.replicate(5))

	genome.evaluator = obj

	pop = ga_parallel_pop(genome,size,server_list)

	print  '########### awaiting evaluation#############'

	pop.initialize()

	print ' evaluation done!'

	print 'best:', pop.best()

	print 'worst',pop.worst()



def gen_pop():

	genome.list_genome.evaluator = objective()

	gene = gene.float_gene((0,2.5))

	genome = genome.list_genome(gene.replicate(5))

	pop = ga_parallel_pop(genome,100,[(host,port),])

	return pop



	import parallel_pop,beowulf,os



def test_pop2(server_list,size=100,wait=.01):

	import hmm_gnm,os

	genome = hmm_gnm.make_genome()

	#pop = ga_parallel_pop(genome,4,server_list)

	global galg

	#genome.target = targets[0]

	pop = ga_parallel_pop(genome,1,server_list)

	galg = hmm_gnm.class_ga(pop)

	galg.settings.update({ 'pop_size':6,'gens':2,'p_mutate':.03,

				    'dbase':os.environ['HOME'] + '/all_lift3', 'p_cross':0.9, 'p_replace':.6,

				    'p_deviation': -.001})

	galg.evolve()



	print  '########### awaiting evaluation#############'

	pop.initialize()

	print ' evaluation done!'

	print 'best:', pop.best()

	print 'worst',pop.worst()

	

import thread

def test():

	host = socket.gethostname()

	port = 8000

	server_list = [(host,port),(host,port+1)]

	for server in server_list:

		host,port = server

		thread.start_new_thread(remote_exec.server,(host,port))

	thread.start_new_thread(test_pop2,(server_list,))		

	

def test2(machines=32,size=100,wait=.01):

	import time

	t1 = time.time()

	#requires that servers are started on beowulf 1 and 2.

	import beowulf

	server_list = beowulf.beowulf.servers[:machines]

	thread.start_new_thread(test_pop,(server_list,size,wait))			

	print 'total time:', time.time()-t1



""" Modified version of build_clib that handles fortran source files.
"""

import os
import string
import sys
import re
from glob import glob
from types import *
from distutils.command.build_clib import build_clib as old_build_clib
from distutils.command.build_clib import show_compilers

from scipy_distutils import log, misc_util
from distutils.dep_util import newer_group
from scipy_distutils.misc_util import filter_sources, \
     has_f_sources, has_cxx_sources

def get_headers(directory_list):
    # get *.h files from list of directories
    headers = []
    for dir in directory_list:
        head = glob(os.path.join(dir,"*.h")) #XXX: *.hpp files??
        headers.extend(head)

    return headers

def get_directories(list_of_sources):
    # get unique directories from list of sources.
    direcs = []
    for file in list_of_sources:
        dir = os.path.split(file)
        if dir[0] != '' and not dir[0] in direcs:
            direcs.append(dir[0])
    return direcs

class build_clib(old_build_clib):

    description = "build C/C++/F libraries used by Python extensions"

    user_options = old_build_clib.user_options + [
        ('fcompiler=', None,
         "specify the Fortran compiler type"),
        ]

    def initialize_options(self):
        old_build_clib.initialize_options(self)
        self.fcompiler = None
        return

    def finalize_options(self):
        old_build_clib.finalize_options(self)
        self.set_undefined_options('build_ext',
                                   ('fcompiler', 'fcompiler'))

        #XXX: This is hackish and probably unnecessary,
        #     could we get rid of this?
        from scipy_distutils import misc_util
        extra_includes = misc_util.get_environ_include_dirs()
        if extra_includes:
            print "XXX: are you sure you'll need PYTHONINCLUDES env. variable??"
        self.include_dirs.extend(extra_includes)

        return

    def have_f_sources(self):
        for (lib_name, build_info) in self.libraries:
            if has_f_sources(build_info.get('sources',[])):
                return 1
        return 0

    def have_cxx_sources(self):
        for (lib_name, build_info) in self.libraries:
            if has_cxx_sources(build_info.get('sources',[])):
                return 1
        return 0

    def run(self):
        if not self.libraries:
            return

        # Make sure that library sources are complete.
        for (lib_name, build_info) in self.libraries:
            if not misc_util.all_strings(build_info.get('sources',[])):
                raise TypeError,'Library "%s" sources contains unresolved'\
                      ' items (call build_src before built_clib).' % (lib_name)

        from distutils.ccompiler import new_compiler
        self.compiler = new_compiler(compiler=self.compiler,
                                     dry_run=self.dry_run,
                                     force=self.force)
        self.compiler.customize(self.distribution,need_cxx=self.have_cxx_sources())

        libraries = self.libraries
        self.libraries = None
        self.compiler.customize_cmd(self)
        self.libraries = libraries

        self.compiler.show_customization()

        if self.have_f_sources():
            from scipy_distutils.fcompiler import new_fcompiler
            self.fcompiler = new_fcompiler(compiler=self.fcompiler,
                                           verbose=self.verbose,
                                           dry_run=self.dry_run,
                                           force=self.force)
            self.fcompiler.customize(self.distribution)
    
            libraries = self.libraries
            self.libraries = None
            self.fcompiler.customize_cmd(self)
            self.libraries = libraries

            self.fcompiler.show_customization()

        self.build_libraries(self.libraries)
        return

    def get_source_files(self):
        from build_ext import is_local_src_dir
        self.check_library_list(self.libraries)
        filenames = []
        def visit_func(filenames,dirname,names):
            if os.path.basename(dirname) in ['CVS','.svn']:
                names[:] = []
                return
            for name in names:
                if name[-1] in "#~":
                    continue
                fullname = os.path.join(dirname,name)
                if os.path.isfile(fullname):
                    filenames.append(fullname)
        for (lib_name, build_info) in self.libraries:
            sources = build_info.get('sources',[])
            sources = filter(lambda s:type(s) is StringType,sources)
            filenames.extend(sources)
            filenames.extend(get_headers(get_directories(sources)))
            depends = build_info.get('depends',[])
            for d in depends:
                if is_local_src_dir(d):
                    os.path.walk(d,visit_func,filenames)
                elif os.path.isfile(d):
                    filenames.append(d)
        return filenames

    def build_libraries(self, libraries):

        compiler = self.compiler
        fcompiler = self.fcompiler

        for (lib_name, build_info) in libraries:
            sources = build_info.get('sources')
            if sources is None or type(sources) not in (ListType, TupleType):
                raise DistutilsSetupError, \
                      ("in 'libraries' option (library '%s'), " +
                       "'sources' must be present and must be " +
                       "a list of source filenames") % lib_name
            sources = list(sources)

            lib_file = compiler.library_filename(lib_name,
                                                 output_dir=self.build_clib)

            depends = sources + build_info.get('depends',[])
            if not (self.force or newer_group(depends, lib_file, 'newer')):
                log.debug("skipping '%s' library (up-to-date)", lib_name)
                continue
            else:
                log.info("building '%s' library", lib_name)

            macros = build_info.get('macros')
            include_dirs = build_info.get('include_dirs')
            extra_postargs = build_info.get('extra_compiler_args') or []

            c_sources, cxx_sources, f_sources, fmodule_sources \
                       = filter_sources(sources)

            if self.compiler.compiler_type=='msvc':
                # this hack works around the msvc compiler attributes
                # problem, msvc uses its own convention :(
                c_sources += cxx_sources
                cxx_sources = []

            if fmodule_sources:
                print 'XXX: Fortran 90 module support not implemented or tested'
                f_sources.extend(fmodule_sources)

            objects = []
            if c_sources:
                log.info("compiling C sources")
                objects = compiler.compile(c_sources,
                                           output_dir=self.build_temp,
                                           macros=macros,
                                           include_dirs=include_dirs,
                                           debug=self.debug,
                                           extra_postargs=extra_postargs)

            if cxx_sources:
                log.info("compiling C++ sources")
                old_compiler = self.compiler.compiler_so[0]
                self.compiler.compiler_so[0] = self.compiler.compiler_cxx[0]

                cxx_objects = compiler.compile(cxx_sources,
                                               output_dir=self.build_temp,
                                               macros=macros,
                                               include_dirs=include_dirs,
                                               debug=self.debug,
                                               extra_postargs=extra_postargs)
                objects.extend(cxx_objects)

                self.compiler.compiler_so[0] = old_compiler

            if f_sources:
                log.info("compiling Fortran sources")
                f_objects = fcompiler.compile(f_sources,
                                              output_dir=self.build_temp,
                                              macros=macros,
                                              include_dirs=include_dirs,
                                              debug=self.debug,
                                              extra_postargs=[])
                objects.extend(f_objects)

            self.compiler.create_static_lib(objects, lib_name,
                                            output_dir=self.build_clib,
                                            debug=self.debug)
        return



# Authors:  Travis Oliphant, Trent Oliphant
# with support from Lee Barford's group at Agilent, Inc.
#

"""This module allows for the loading of an array from an ASCII
Text File

"""

__all__ = ['read_array', 'write_array']
import Numeric
import scipy_base
from scipy_base import array, take, concatenate, Float, asarray, real, imag
import types, re, copy, sys
import numpyio
default = None
_READ_BUFFER_SIZE = 1024*1024
#_READ_BUFFER_SIZE = 1000
#_READ_BUFFER_SIZE = 160

# ASCII Text object stream with automatic (un)compression and URL access.
#
# Adapted from
#    TextFile class Written by: Konrad Hinsen <hinsen@cnrs-orleans.fr>
#
# Written by Travis Oliphant and Trent Oliphant
#   with support from Agilent, Inc.
# 

import os, sys

def convert_separator(sep):
    newsep = ''
    for k in sep:
        if k in '.^$*+?{[\\|()':
            newsep = newsep + '\\' + k
        else:
            newsep = newsep + k
    return newsep

def build_numberlist(lines):
    if lines is default:
        linelist = [-1]
    else:
        linelist = []
        errstr = "Argument lines must be a sequence of integers and/or range tuples."
        try:
            for num in lines[:-1]:  # handle all but last element 
                if type(num) not in [types.IntType, types.TupleType]:
                    raise ValueError, errstr
                if isinstance(num, types.IntType):
                    linelist.append(num)
                else:
                    if not 1 < len(num) < 4:
                        raise ValueError, "Tuples must be valid range tuples."                        
                    linelist.extend(range(*num))
        except TypeError:
            raise ValueError, errstr
        num = lines[-1]
        if type(num) is types.IntType:
            linelist.append(num)
        elif type(num) is types.TupleType:
            if [types.IntType]*len(num) != map(type, num):
                if len(num) > 1 and num[1] is not None:                        
                    raise ValueError, errstr
            if len(num) == 1:
                linelist.extend([num[0],-1])
            elif len(num) == 2:
                if num[1] is None:
                    linelist.extend([num[0], -1])
                else:
                    linelist.extend(range(*num))
            elif len(num) == 3:
                if num[1] is None:
                    linelist.extend([num[0], -num[2]])
                else:
                    linelist.extend(range(*num))
            else:
                raise ValueError, errstr
    return linelist

def get_open_file(fileobject, mode='rb'):
    if isinstance(fileobject, types.StringType):
        fileobject = os.path.expanduser(fileobject)
        if mode[0]=='r' and not os.path.exists(fileobject):
            raise IOError, (2, 'No such file or directory: '
                            + fileobject)
        else:
            try:
                file = open(fileobject, mode)
            except IOError, details:
                file = None
                if type(details) == type(()):
                    details = details + (fileobject,)
                raise IOError, details
    else:
        file = fileobject

    return file
            

class ascii_stream:
    """Text files with line iteration

    Ascii_stream instances can be used like normal read-only file objects
    (i.e. by calling readline() and readlines()), but can
    also be used as sequences of lines in for-loops.

    Finally, ascii_stream objects accept file names that start with '~' or
    '~user' to indicate a home directory(for reading only).

    Constructor: ascii_stream(|fileobject|, |lines|,|comment|),
    where |fileobject| is either an open python file object or 
    the name of the file, |lines| is a sequence of integers
    or tuples(indicating ranges) of lines to be read, |comment| is the
    comment line identifier """

    def __init__(self, fileobject, lines=default, comment="#",
                 linesep='\n'):
        if not isinstance(comment, types.StringType):
            raise ValueError, "Comment must be a string."
        self.linelist = build_numberlist(lines)
        self.comment = comment
        self.lencomment = len(comment)
        self.file = get_open_file(fileobject, mode='r')
        self._pos = self.file.tell()
        self._lineindex = 0
        if self.linelist[-1] < 0:
            self._linetoget = self.linelist[-1]
        else:
            self._linetoget = 0
        self._oldbuflines = 0
        self._linesplitter = linesep
        self._buffer = self.readlines(_READ_BUFFER_SIZE)
        self._totbuflines = len(self._buffer)

    def readlines(self, sizehint):
        buffer = self.file.read(sizehint)
        lines = buffer.split(self._linesplitter)
        if len(buffer) < sizehint:  # EOF
            if buffer == '':
                return []
            else:
                return lines
        else:
            if len(lines) < 2:
                raise ValueError, "Buffer size too small."
            backup = len(lines[-1])
            self.file.seek(-backup, 1)
            return lines[:-1]

    def __del__(self):
        if hasattr(self.file,'close'):
            self.file.close()

    def __getitem__(self, item):
        while 1:
            line = self.readnextline()
            if line is None:
                raise IndexError
            if len(line) < self.lencomment or line[:self.lencomment] != self.comment:
                break
        return line
        
    def readnextline(self):
        if self.linelist[self._lineindex] >= 0:
            self._linetoget = self.linelist[self._lineindex]
            self._lineindex += 1
        else:
            self._linetoget = self._linetoget - self.linelist[self._lineindex]
        while self._linetoget >= self._totbuflines:
            self._buffer = self.readlines(_READ_BUFFER_SIZE)
            self._oldbuflines = self._totbuflines
            self._totbuflines += len(self._buffer)
            if (self._totbuflines == self._oldbuflines):
                return None
        line = self._buffer[self._linetoget - self._oldbuflines]
        return line    

    def close(self):
        self.file.close()

    def flush(self):
        self.file.flush()


def move_past_spaces(firstline):
    ind = 0
    firstline = firstline.lstrip()
    while firstline[ind] not in [' ','\n','\t','\v','\f','\r']:
        ind += 1
    return firstline[ind:], ind


def extract_columns(arlist, collist, atype, missing):
    if collist[-1] < 0:
        if len(collist) == 1:
            toconvlist = arlist[::-collist[-1]]
        else:
            toconvlist = take(arlist,collist[:-1])
            toconvlist = concatenate((toconvlist,
                                      arlist[(collist[-2]-collist[-1])::(-collist[-1])]))
    else:
        toconvlist = take(arlist, collist)

    return numpyio.convert_objectarray(toconvlist, atype, missing)
    

# Given a string representing one line, a separator tuple, a list of 
#  columns to read for each element of the atype list and a missing
#  value to insert when conversion fails.

# Regular expressions for detecting complex numbers and for dealing
#  with spaces between the real and imaginary parts

import re
_obj = re.compile(r"""
      ([0-9.eE]+)            # Real part
      ([\t ]*)               # Space between real and imaginary part
      ([+-])                 # +/- sign
      ([\t ]*)               # 0 or more spaces
      (([0-9.eE]+[iIjJ])
      |([iIjJ][0-9.eE]+))    # Imaginary part
      """, re.VERBOSE)

_not_warned = 1
def process_line(line, separator, collist, atype, missing):
    global _not_warned
    strlist = []
    line = _obj.sub(r"\1\3\5",line)  # remove spaces between real
                                     # and imaginary parts of complex numbers

    if _not_warned:
        warn = 0
        if (_obj.search(line) is not None):
            warn = 1
            for k in range(len(atype)):
                if atype[k] in scipy_base.typecodes['Complex']:
                    warn = 0
        if warn:
            scipy_base.disp("Warning: Complex data detected, but no requested typecode was complex.")
            _not_warned = 0
    for mysep in separator[:-1]:
        if mysep is None:
            newline, ind = move_past_spaces(line)
            strlist.append(line[:ind])
            line = newline
        else:
            ind = line.find(mysep)
            strlist.append(line[:ind])
            line = line[ind+len(mysep):]
    strlist.extend(line.split(separator[-1]))
    arlist = array(strlist,'O')
    N = len(atype)
    vals = [None]*N
    for k in range(len(atype)):
        vals[k] = extract_columns(arlist, collist[k], atype[k], missing)
    return vals

def getcolumns(stream, columns, separator):
    global _not_warned
    comment = stream.comment
    lenc = stream.lencomment
    k, K = stream.linelist[0], len(stream._buffer)
    while k < K:
        firstline = stream._buffer[k]
        if firstline != '' and firstline[:lenc] != comment:
            break
        k = k + 1
    if k == K:
        raise ValueError, "First line to read not within %d lines of top." % K
    firstline = stream._buffer[k]
    N = len(columns)    
    collist = [None]*N
    colsize = [None]*N
    for k in range(N):
        collist[k] = build_numberlist(columns[k])
    _not_warned = 0
    val = process_line(firstline, separator, collist, [Numeric.Float]*N, 0)
    for k in range(N):
        colsize[k] = len(val[k])
    return colsize, collist

def convert_to_equal_lists(cols, atype):
    if not isinstance(cols, types.ListType):
        cols = [cols]
    if not isinstance(atype, types.ListType):
        atype = [atype]
    N = len(cols) - len(atype)
    if N > 0:
        atype.extend([atype[-1]]*N)
    elif N < 0:
        cols.extend([cols[-1]]*(-N))
    return cols, atype


def read_array(fileobject, separator=default, columns=default, comment="#",
               lines=default, atype=Numeric.Float, linesep='\n',
               rowsize=10000, missing=0):
    """Return an array or arrays from ascii_formatted data in |fileobject|.

    Inputs:

      fileobject -- An open file object or a string for a valid filename.
                    The string can be prepended by "~/" or "~<name>/" to
                    read a file from the home directory.
      separator -- a string or a tuple of strings to indicate the column
                   separators.  If the length of the string tuple is less
                   than the total number of columns, then the last separator
                   is assumed to be the separator for the rest of the columns.
      columns -- a tuple of integers and range-tuples which describe the
                 columns to read from the file.  A negative entry in the
                 last column specifies the negative skip value to the end.
                 Example:  columns=(1, 4, (5, 9), (11, 15, 3), 17, -2)
                         will read [1,4,5,6,7,8,11,14,17,19,21,23,...]
                 If multiple arrays are to be returned, then this argument
                 should be an ordered list of such tuples.  There should be
                 one entry in the list for each arraytype in the atype list.
      lines   -- a tuple with the same structure as columns which indicates
                 the lines to read. 
      comment -- the comment character (line will be ignored even if it is
                 specified by the lines tuple)
      linesep -- separator between rows.
      missing -- value to insert in array when conversion to number fails.
      atype -- the typecode of the output array.  If multiple outputs are
               desired, then this should be a list of typecodes.  The columns
               to fill the array represented by the given typecode is
               determined from the columns argument.  If the length of atype
               does not match the length of the columns list, then, the
               smallest one is expanded to match the largest by repeatedly
               copying the last entry.
      rowsize -- the allocation row size (array grows by this amount as
               data is read in).

    Output -- the 1 or 2d array, or a tuple of output arrays of different
              types, sorted in order of the first column to be placed
              in the output array. 

    """

    global _not_warned
    # Make separator into a tuple of separators.
    if type(separator) in [types.StringType, type(default)]:
        sep = (separator,)
    else:
        sep = tuple(separator)
    # Create ascii_object from |fileobject| argument.
    ascii_object = ascii_stream(fileobject, lines=lines, comment=comment, linesep=linesep)
    columns, atype = convert_to_equal_lists(columns, atype)
    numout = len(atype)
    # Get the number of columns to read and expand the columns argument
    colsize, collist = getcolumns(ascii_object, columns, sep)
    # Intialize the output arrays
    outrange = range(numout)
    outarr = []
    typecodes = "".join(Numeric.typecodes.values())
    for k in outrange:
        if not atype[k] in typecodes:
            raise ValueError, "One of the array types is invalid, k=%d" % k
        outarr.append(Numeric.zeros((rowsize, colsize[k]),atype[k]))
    row = 0
    block_row = 0
    _not_warned = 1
    for line in ascii_object:
        if line.strip() == '':
            continue
        vals = process_line(line, sep, collist, atype, missing)
        for k in outrange:
            outarr[k][row] = vals[k]
        row += 1
        block_row += 1
        if block_row >= rowsize:
            for k in outrange:
                outarr[k].resize((outarr[k].shape[0] + rowsize,colsize[k]))
            block_row = 0
    for k in outrange:
        if outarr[k].shape[0] != row:
            outarr[k].resize((row,colsize[k]))
        a = outarr[k]            
        if a.shape[0] == 1 or a.shape[1] == 1:
            outarr[k] = Numeric.ravel(a)
    if len(outarr) == 1:
        return outarr[0]
    else:
        return tuple(outarr)


# takes 1-d array and returns a string
def str_array(arr, precision=5,col_sep=' ',row_sep="\n",ss=0):
    thestr = []
    arr = asarray(arr)
    N,M = arr.shape
    thistype = arr.typecode()
    nofloat = (thistype in '1silbwu') or (thistype in 'Oc')
    cmplx = thistype in 'FD'
    fmtstr = "%%.%de" % precision
    cmpnum = pow(10.0,-precision)
    for n in xrange(N):
        theline = []
        for m in xrange(M):
            val = arr[n,m]
            if ss and abs(val) < cmpnum:
                val = 0*val
            if nofloat or val==0:
                thisval = str(val)
            elif cmplx:
                rval = real(val)
                ival = imag(val)
                thisval = eval('fmtstr % rval')
                if (ival >= 0):
                    istr = eval('fmtstr % ival')
                    thisval = '%s+j%s' % (thisval, istr) 
                else:
                    istr = eval('fmtstr % abs(ival)')                    
                    thisval = '%s-j%s' % (thisval, istr)
            else:
                thisval = eval('fmtstr % val')
            theline.append(thisval)
        strline = col_sep.join(theline)
        thestr.append(strline)
    return row_sep.join(thestr)
        

def write_array(fileobject, arr, separator=" ", linesep='\n',
                precision=5, suppress_small=0, keep_open=0):
    """Write a rank-2 or less array to file represented by fileobject.

    Inputs:

      fileobject -- An open file object or a string to a valid filename.
      arr -- The array to write.
      separator -- separator to write between elements of the array.
      linesep -- separator to write between rows of array
      precision -- number of digits after the decimal place to write.
      suppress_small -- non-zero to round small numbers down to 0.0
      keep_open = non-zero to return the open file, otherwise, the file is closed.
    Outputs:

      file -- The open file (if keep_open is non-zero)
    """
    file = get_open_file(fileobject, mode='wa')
    rank = Numeric.rank(arr)
    if rank > 2:
        raise ValueError, "Can-only write up to 2-D arrays."

    if rank == 0:
        h = 1
        arr = Numeric.reshape(arr, (1,1))
    elif rank == 1:
        h = Numeric.shape(arr)[0]
        arr = Numeric.reshape(arr, (h,1))
    else:
        h = Numeric.shape(arr)[0]
        arr = asarray(arr)

    for ch in separator:
        if ch in '0123456789-+FfeEgGjJIi.':
            raise ValueError, "Bad string for separator"
        
    astr = str_array(arr, precision=precision,
                     col_sep=separator, row_sep=linesep,
                     ss = suppress_small)
    file.write(astr)
    file.write('\n')
    if keep_open:
        return file
    else:
        if file is sys.stdout or file is sys.stderr:
            return
        file.close()
    return



"""
Discrete Fourier Transforms - helper.py
"""
# Created by Pearu Peterson, September 2002

__all__ = ['fftshift','ifftshift','fftfreq','rfftfreq']

from scipy_base import asarray, concatenate, arange, take, array
import types

def fftshift(x,axes=None):
    """ fftshift(x, axes=None) -> y

    Shift zero-frequency component to center of spectrum.

    This function swaps half-spaces for all axes listed (defaults to all).

    Notes:
      If len(x) is even then the Nyquist component is y[0].
    """
    tmp = asarray(x)
    ndim = len(tmp.shape)
    if axes is None:
        axes = range(ndim)
    y = tmp
    for k in axes:
        n = tmp.shape[k]
        p2 = (n+1)/2
        mylist = concatenate((arange(p2,n),arange(p2)))
        y = take(y,mylist,k)
    return y


def ifftshift(x,axes=None):
    """ ifftshift(x,axes=None) - > y

    Inverse of fftshift.
    """
    tmp = asarray(x)
    ndim = len(tmp.shape)
    if axes is None:
        axes = range(ndim)
    y = tmp
    for k in axes:
        n = tmp.shape[k]
        p2 = n-(n+1)/2
        mylist = concatenate((arange(p2,n),arange(p2)))
        y = take(y,mylist,k)
    return y

def fftfreq(n,d=1.0):
    """ fftfreq(n, d=1.0) -> f

    DFT sample frequencies

    The returned float array contains the frequency bins in
    cycles/unit (with zero at the start) given a window length n and a
    sample spacing d:

      f = [0,1,...,n/2-1,-n/2,...,-1]/(d*n)         if n is even
      f = [0,1,...,(n-1)/2,-(n-1)/2,...,-1]/(d*n)   if n is odd
    """
    assert isinstance(n,types.IntType)
    k = range(0,(n-1)/2+1)+range(-(n/2),0)
    return array(k,'d')/(n*d)

def rfftfreq(n,d=1.0):
    """ rfftfreq(n, d=1.0) -> f

    DFT sample frequencies (for usage with rfft,irfft).

    The returned float array contains the frequency bins in
    cycles/unit (with zero at the start) given a window length n and a
    sample spacing d:

      f = [0,1,1,2,2,...,n/2-1,n/2-1,n/2]/(d*n)   if n is even
      f = [0,1,1,2,2,...,n/2-1,n/2-1,n/2,n/2]/(d*n)   if n is odd
    """
    assert isinstance(n,types.IntType)
    return array(range(1,n+1),'i')/2/float(n*d)




# Author: Travis Oliphant

__all__ = ['odeint']

import _odepack

_msgs = {2: "Integration successful.",
         -1: "Excess work done on this call (perhaps wrong Dfun type).",
         -2: "Excess accuracy requested (tolerances too small).",
         -3: "Illegal input detected (internal error).",
         -4: "Repeated error test failures (internal error).",
         -5: "Repeated convergence failures (perhaps bad Jacobian or tolerances).",
         -6: "Error weight became zero during problem.",
         -7: "Internal workspace insufficient to finish (internal error)."
         }

def odeint(func, y0, t, args=(), Dfun=None, col_deriv=0, full_output=0,
           ml=None, mu=None, rtol=None, atol=None, tcrit=None, h0=0.0,
           hmax=0.0, hmin=0.0, ixpr=0, mxstep=0, mxhnil=0, mxordn=12,
           mxords=5, printmessg=0):

    """Integrate a system of ordinary differential equations.

    Description:

      Solve a system of ordinary differential equations Using lsoda from the
      FORTRAN library odepack.
      
      Solves the initial value problem for stiff or non-stiff systems
      of first order ode-s:
           dy/dt = func(y,t0,...) where y can be a vector.

    Inputs:

      func -- func(y,t0,...) computes the derivative of y at t0.
      y0   -- initial condition on y (can be a vector).
      t    -- a sequence of time points for which to solve for y.  The intial 
              value point should be the first element of this sequence.
      args -- extra arguments to pass to function.
      Dfun -- the gradient (Jacobian) of func (same input signature as func).
      col_deriv -- non-zero implies that Dfun defines derivatives down
                   columns (faster), otherwise Dfun should define derivatives
                   across rows.
      full_output -- non-zero to return a dictionary of optional outputs as
                     the second output.
      printmessg -- print the convergence message.

    Outputs: (y, {infodict,})

      y -- a rank-2 array containing the value of y in each row for each
           desired time in t (with the initial value y0 in the first row).

      infodict -- a dictionary of optional outputs:
        'hu'    : a vector of step sizes successfully used for each time step.
        'tcur'  : a vector with the value of t reached for each time step.
                  (will always be at least as large as the input times).
        'tolsf' : a vector of tolerance scale factors, greater than 1.0,
                  computed when a request for too much accuracy was detected.
        'tsw'   : the value of t at the time of the last method switch
                  (given for each time step).
        'nst'   : the cumulative number of time steps.
        'nfe'   : the cumulative number of function evaluations for eadh
                  time step.
        'nje'   : the cumulative number of jacobian evaluations for each
                  time step.
        'nqu'   : a vector of method orders for each successful step.
        'imxer' : index of the component of largest magnitude in the
                   weighted local error vector (e / ewt) on an error return.
        'lenrw' : the length of the double work array required.
        'leniw' : the length of integer work array required.
        'mused' : a vector of method indicators for each successful time step:
                  1 -- adams (nonstiff)
                  2 -- bdf (stiff)

    Additional Inputs:

      ml, mu -- If either of these are not-None or non-negative, then the
                Jacobian is assumed to be banded.  These give the number of
                lower and upper non-zero diagonals in this banded matrix.
                For the banded case, Dfun should return a matrix whose
                columns contain the non-zero bands (starting with the
                lowest diagonal).  Thus, the return matrix from Dfun should
                have shape len(y0) x (ml + mu + 1) when ml >=0 or mu >=0
      rtol -- The input parameters rtol and atol determine the error
      atol    control performed by the solver.  The solver will control the
              vector, e, of estimated local errors in y, according to an
              inequality of the form
                   max-norm of (e / ewt) <= 1
              where ewt is a vector of positive error weights computed as
                   ewt = rtol * abs(y) + atol
              rtol and atol can be either vectors the same length as y or
              scalars.
      tcrit -- a vector of critical points (e.g. singularities) where
               integration care should be taken.

       (For the next inputs a zero default means the solver determines it).

      h0 -- the step size to be attempted on the first step.
      hmax -- the maximum absolute step size allowed.
      hmin -- the minimum absolute step size allowed.
      ixpr -- non-zero to generate extra printing at method switches.
      mxstep -- maximum number of (internally defined) steps allowed
                for each integration point in t.
      mxhnil -- maximum number of messages printed.
      mxordn -- maximum order to be allowed for the nonstiff (Adams) method.
      mxords -- maximum order to be allowed for the stiff (BDF) method.

    """

    if ml is None:
        ml = -1 # changed to zero inside function call
    if mu is None:
        mu = -1 # changed to zero inside function call
    t = t.copy()
    output = _odepack.odeint(func, y0, t, args, Dfun, col_deriv, ml, mu,
                             full_output, rtol, atol, tcrit, h0, hmax, hmin,
                             ixpr, mxstep, mxhnil, mxordn, mxords)
    if output[-1] < 0:
        print _msgs[output[-1]]
        print "Run with full_output = 1 to get quantitative information."
    else:
        if printmessg:
            print _msgs[output[-1]]
        
    if full_output:
        output[1]['message'] = _msgs[output[-1]]

    output = output[:-1]
    if len(output) == 1:
        return output[0]
    else:
        return output






"""
    Allows interpreter and GUI windows to co-exist peacefully.

    gui_thread runs wxPython in a back ground thread and allows you 
    to create and manipulate wxPython windows from the command line.

    NEW WAY:
    An example usage follows::
           >> import gui_thread
           >> gui_thread.start()
           >> from gui_thread.examples import SimpleFrame
           >> a = SimpleFrame()
           >> a.Show(1)
           # a window with a button should pop up...
           >> a.SetTitle('bob')
           >> a.GetTitle()
           'bob'
           >> a.GetSize()
           (350,200)
           >> a.Blue()
           # window button should have turned blue.
           >> b = SimpleFrame()
           >> b.Show(1)
           # a 2nd window appears

    OLD WAY:
    An example usage follows::
           >> import gui_thread
           >> gui_thread.start(use_main=1)
           <wait several seconds while wxPython starts>
           wxPython imported           
           >> from gui_thread.examples import SimpleFrame
           >> prxyMyFrame = gui_thread.register(SimpleFrame)
           >> a = prxyMyFrame()
           >> a.Show(1)
           # a window with a button should pop up...
           >> a.SetTitle('bob')
           >> a.GetTitle()
           'bob'
           >> a.GetSize()
           (350,200)
           >> a.Blue()
           # window button should have turned blue.
           >> b = prxyMyFrame()
           >> b.Show(1)
           # a 2nd window appears
        
        Modules based on gui_thread will also run in wxPython applications
        without modification.  So, by using gui_thread, you can create
        viewers for common objects that are usable from the standard
        python interpreter, PythonWin (on MS), and wxPython applications.
        Since Tkinter and wxPython currently do not coexist well together,
        IDLE is not compatible with gui_thread.        
        
        gui_thread should always be the first module you import into an 
        application, unless it is a wxPython app.  In that case, it can
        be imported at any time.  When using it in interactive mode, make
        wait until the "wxPython Imported" message appears before importing
        any other modules that use wxPython.
"""

# New hooks for importing wxPython to its own thread
from wxPython_thread import wxPython_thread

_use_main = 0
def start(extra=None,use_main=0):
    global _use_main
    _use_main = use_main
    if _use_main:
        import main
        main.start(extra)
    else:
        wxPython_thread()
    return

def start_up(src_modname,dst_modname,extra=None):
    global _use_main
    if _use_main:
        import main
        main.start_up(src_modname,dst_modname,extra)
    return

def register(wx_class):
    global _use_main
    if _use_main:
        import main
        return main.register(wx_class)
    return wx_class



# Author: Travis Oliphant

from MLab import squeeze
from scipy_base import *
from scipy_base.fastumath import *
import numpyio
import struct, os, sys
import types
try:
    import scipy.sparse
    have_sparse = 1
except ImportError:
    have_sparse = 0

if sys.version_info[0] < 2 or sys.version_info[1] < 3:
    False = 0
    True = 1

__all__ = ['fopen','loadmat','savemat']

def getsize_type(mtype):
    if mtype in ['b','uchar','byte','unsigned char','integer*1', 'int8']:
        mtype = 'b'
    elif mtype in ['c', 'char','char*1']:
        mtype = 'c'
    elif mtype in ['1','schar', 'signed char']:
        mtype = '1'
    elif mtype in ['s','short','int16','integer*2']:
        mtype = 's'
    elif mtype in ['w','ushort','uint16','unsigned short']:
        mtype = 'w'
    elif mtype in ['i','int']:
        mtype = 'i'
    elif mtype in ['u','uint','uint32','unsigned int']:
        mtype = 'u'
    elif mtype in ['l','long','int32','integer*4']:
        mtype = 'l'
    elif mtype in ['f','float','float32','real*4', 'real']:
        mtype = 'f'
    elif mtype in ['d','double','float64','real*8', 'double precision']:
        mtype = 'd'
    elif mtype in ['F','complex float','complex*8','complex64']:
        mtype = 'F'
    elif mtype in ['D','complex*16','complex128','complex','complex double']:
        mtype = 'D'
    else:
        raise TypeError, 'Bad datatype -- ' + mtype

    argout = (array(0,mtype).itemsize(),mtype)
    return argout

class fopen:
    """Class for reading and writing binary files into Numeric arrays.

    Inputs:

      file_name -- The complete path name to the file to open.
      permission -- Open the file with given permissions: ('r', 'w', 'a')
                    for reading, writing, or appending.  This is the same
                    as the mode argument in the builtin open command.
      format -- The byte-ordering of the file:
                (['native', 'n'], ['ieee-le', 'l'], ['ieee-be', 'b']) for
                native, little-endian, or big-endian respectively.

    Methods:

      read -- read data from file and return Numeric array
      write -- write to file from Numeric array
      fort_read -- read Fortran-formatted binary data from the file.
      fort_write -- write Fortran-formatted binary data to the file.
      rewind -- rewind to beginning of file
      size -- get size of file
      seek -- seek to some position in the file
      tell -- return current position in file
      close -- close the file

    Attributes (Read only):

      bs -- non-zero if byte-swapping is performed on read and write.
      format -- 'native', 'ieee-le', or 'ieee-be'
      fid -- the file object
      closed -- non-zero if the file is closed.
      mode -- permissions with which this file was opened
      name -- name of the file
      
    """
    
    def __init__(self,file_name,permission='rb',format='n'):
        if 'b' not in permission: permission += 'b'
        if type(file_name) in (types.StringType, types.UnicodeType):
            self.__dict__['fid'] = open(file_name,permission)
        elif 'fileno' in file_name.__methods__:  # first argument is an open file
            self.__dict__['fid'] = file_name 
        if format in ['native','n','default']:
            self.__dict__['bs'] = 0
            self.__dict__['format'] = 'native'
        elif format in ['ieee-le','l','little-endian','le']:
            self.__dict__['bs'] = not LittleEndian
            self.__dict__['format'] = 'ieee-le'
        elif format in ['ieee-be','b','big-endian','be']:
            self.__dict__['bs'] = LittleEndian
            self.__dict__['format'] = 'ieee-be'
        else:
            raise ValueError, "Unrecognized format: " + format

        self.__dict__['seek'] = self.fid.seek
        self.__dict__['tell']= self.fid.tell
        self.__dict__['close'] = self.fid.close
        self.__dict__['fileno'] = self.fid.fileno
        self.__dict__['mode'] = self.fid.mode
        self.__dict__['closed'] = self.fid.closed
        self.__dict__['name'] = self.fid.name

    def __setattr__(self, attribute):
        raise SyntaxError, "There are no user-settable attributes."            

    def __del__(self):
        try:
            self.fid.close()
        except:
            pass

    def setformat(self, format):
        if format in ['native','n','default']:
            self.__dict__['bs'] = False
            self.__dict__['format'] = 'native'
        elif format in ['ieee-le','l','little-endian','le']:
            self.__dict__['bs'] = not LittleEndian
            self.__dict__['format'] = 'ieee-le'
        elif format in ['ieee-be','b','big-endian','be']:
            self.__dict__['bs'] = LittleEndian
            self.__dict__['format'] = 'ieee-be'
        else:
            raise ValueError, "Unrecognized format: " + format
        return
        
    def write(self,data,mtype=None,bs=None):
        """Write to open file object the flattened Numeric array data.

        Inputs:

          data -- the Numeric array to write.
          mtype -- a string indicating the binary type to write.
                   The default is the type of data. If necessary a cast is made.
                   unsigned byte  : 'b', 'uchar', 'byte' 'unsigned char', 'int8',
                                    'integer*1'
                   character      : 'c', 'char', 'char*1'
                   signed char    : '1', 'schar', 'signed char'
                   short          : 's', 'short', 'int16', 'integer*2'
                   unsigned short : 'w', 'ushort','uint16','unsigned short'
                   int            : 'i', 'int'
                   unsigned int   : 'u', 'uint32','uint','unsigned int'
                   long           : 'l', 'long', 'int32', 'integer*4'
                   float          : 'f', 'float', 'float32', 'real*4'
                   double         : 'd', 'double', 'float64', 'real*8'
                   complex float  : 'F', 'complex float', 'complex*8', 'complex64'
                   complex double : 'D', 'complex', 'complex double', 'complex*16',
                                    'complex128'
        """
        if bs is None:
            bs = self.bs
        else:
            bs = (bs == 1)
        data = asarray(data)
        if mtype is None:
            mtype = data.typecode()
        howmany,mtype = getsize_type(mtype)
        count = product(data.shape)
        numpyio.fwrite(self.fid,count,data,mtype,bs)
        return 

    fwrite = write

    def read(self,count,stype,rtype=None,bs=None,c_is_b=0):
        """Read data from file and return it in a Numeric array.

        Inputs:

          count -- an integer specifying the number of elements of type
                   stype to read or a tuple indicating the shape of
                   the output array.
          stype -- The data type of the stored data (see fwrite method).
          rtype -- The type of the output array.  Same as stype if None.
          bs -- Whether or not to byteswap (or use self.bs if None)
          c_is_b --- If non-zero then the count is an integer
                   specifying the total number of bytes to read
                   (must be a multiple of the size of stype).

        Outputs: (output,)

          output -- a Numeric array of type rtype.
        """
        if bs is None:
            bs = self.bs
        else:
            bs = (bs == 1)
        howmany,stype = getsize_type(stype)
        shape = None        
        if c_is_b:
            if count % howmany != 0:
                raise ValueError, "When c_is_b is non-zero then " \
                      "count is bytes\nand must be multiple of basic size."
            count = count / howmany
        elif type(count) in [types.TupleType, types.ListType]:
            shape = list(count)
            # allow -1 to specify unknown dimension size as in reshape
            minus_ones = shape.count(-1)
            if minus_ones == 0:
                count = product(shape)
            elif minus_ones == 1:
                now = self.fid.tell()
                self.fid.seek(0,2)
                end = self.fid.tell()
                self.fid.seek(now)
                remaining_bytes = end - now
                know_dimensions_size = -product(count) * getsize_type(stype)[0]
                unknown_dimension_size, illegal = divmod(remaining_bytes,
                                                         know_dimensions_size)
                if illegal:
                    raise ValueError("unknown dimension doesn't match filesize")
                shape[shape.index(-1)] = unknown_dimension_size
                count = product(shape)
            else:
                raise ValueError(
                    "illegal count; can only specify one unknown dimension")
            shape = tuple(shape)
        if rtype is None:
            rtype = stype
        else:
            howmany,rtype = getsize_type(rtype)
        if count == 0:
            return zeros(0,rtype)
        retval = numpyio.fread(self.fid, count, stype, rtype, bs)
        if len(retval) == 1:
            retval = retval[0]
        if shape is not None:
            retval = resize(retval, shape)
        return retval

    fread = read

    def rewind(self,howmany=None):
        """Rewind a file to it's beginning or by a specified amount.
        """
        if howmany is None:
            self.seek(0)
        else:
            self.seek(-howmany,1)

    def size(self):
        """Return the size of the file.
        """
        try:
            sz = self.thesize
        except AttributeError:            
            curpos = self.tell()
            self.fid.seek(0,2)
            sz = self.fid.tell()
            self.fid.seek(curpos)
            self.__dict__['thesize'] = sz
        return sz

    def fort_write(self,fmt,*args):
        """Write a Fortran binary record.

        Inputs:

          fmt -- If a string then it represents the same format string as
                 used by struct.pack.  The remaining arguments are passed
                 to struct.pack.
                 
                 If fmt is an array, then this array will be written as
                 a Fortran record using the output type args[0].
                 
          *args -- Arguments representing data to write.
        """
        if self.format == 'ieee-le':
            nfmt = "<i"
        elif self.format == 'ieee-be':
            nfmt = ">i"
        else:
            nfmt = "i"
        if type(fmt) in (types.StringType, types.UnicodeType):
            if self.format == 'ieee-le':
                fmt = "<"+fmt
            elif self.format == 'ieee-be':
                fmt = ">"+fmt
            str = apply(struct.pack,(fmt,)+args)
            strlen = struct.pack(nfmt,len(str))
            self.fid.write(strlen)
            self.fid.write(str)
            self.fid.write(strlen)
        elif type(fmt) == type(array([0])):
            if len(args) > 0:
                sz,mtype = getsize_type(args[0])
            else:
                sz,mtype = getsize_type(fmt.typecode())
            count = product(fmt.shape)
            strlen = struct.pack(nfmt,count*sz)
            self.fid.write(strlen)
            numpyio.fwrite(self.fid,count,fmt,mtype,self.bs)
            self.fid.write(strlen)
        else:
            raise TypeError, "Unknown type in first argument"

    def fort_read(self,fmt,dtype=None):
        """Read a Fortran binary record.

        Inputs:

          fmt -- If dtype is not given this represents a struct.pack
                 format string to interpret the next record.  Otherwise this
                 argument is ignored.
          dtype -- If dtype is not None, then read in the next record as
                   an array of type dtype.

        Outputs: (data,)

          data -- If dtype is None, then data is a tuple containing the output
                  of struct.unpack on the next Fortan record.
                  If dtype is a datatype string, then the next record is
                  read in as a 1-D array of type datatype.
        """
        lookup_dict = {'ieee-le':"<",'ieee-be':">",'native':''}
        if dtype is None:
            fmt = lookup_dict[self.format] + fmt
            numbytes = struct.calcsize(fmt)
            nn = struct.calcsize("i");
            if (self.fid.read(nn) == ''):
                raise ValueError, "Unexpected end of file..."
            strdata = self.fid.read(numbytes)
            if strdata == '':
                raise ValueError, "Unexpected end of file..."
            data = struct.unpack(fmt,strdata)
            if (self.fid.read(nn) == ''):
                raise ValueError, "Unexpected end of file..."
            return data
        else:  # Ignore format string and read in next record as an array.
            fmt = lookup_dict[self.format] + "i"
            nn = struct.calcsize(fmt)
            nbytestr = self.fid.read(nn)
            if nbytestr == '':
                raise ValueError, "Unexpected end of file..."
            nbytes = struct.unpack(fmt,nbytestr)[0]
            howmany, dtype = getsize_type(dtype)
            ncount = nbytes / howmany
            if ncount*howmany != nbytes:
                self.rewind(4)
                raise ValueError, "A mismatch between the type requested and the data stored."
            if ncount < 0:
                raise ValueError, "Negative number of bytes to read:\n    file is probably not opened with correct endian-ness."
            if ncount == 0:
                raise ValueError, "End of file?  Zero-bytes to read."
            retval = numpyio.fread(self.fid, ncount, dtype, dtype, self.bs)
            if len(retval) == 1:
                retval = retval[0]
            if (self.fid.read(nn) == ''):
                raise ValueError, "Unexpected end of file..."
            return retval


#### MATLAB Version 5 Support ###########

# Portions of code borrowed and (heavily) adapted
#    from matfile.py by Heiko Henkelmann

## Notice in matfile.py file

# Copyright (c) 2003 Heiko Henkelmann

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.


class mat_struct:    # dummy structure holder
    pass

class mat_obj:    # dummy object holder
    pass

miINT8 = 1
miUINT8 = 2
miINT16 = 3
miUINT16 = 4
miINT32 = 5
miUINT32 = 6
miSINGLE = 7
miDOUBLE = 9
miINT64 =12
miUINT64 = 13
miMATRIX = 14

miNumbers = (
    miINT8,
    miUINT8,
    miINT16,
    miUINT16,
    miINT32,
    miUINT32,
    miSINGLE,
    miDOUBLE,
    miINT64,
    miUINT64,
    )

miDataTypes = {
    miINT8 : ('miINT8', 1,'1'),
    miUINT8 : ('miUINT8', 1,'b'),
    miINT16 : ('miINT16', 2,'s'),
    miUINT16 :('miUINT16',2,'w'),
    miINT32 : ('miINT32',4,'l'),
    miUINT32 : ('miUINT32',4,'u'),
    miSINGLE : ('miSINGLE',4,'f'),
    miDOUBLE : ('miDOUBLE',8,'d'),
    miINT64 : ('miINT64',8,'c'),   # Handle 64 bit integers as string data.
    miUINT64 : ('miUINT64',8,'c'),
    miMATRIX : ('miMATRIX',0,None),
    }
    
mxCELL_CLASS = 1
mxSTRUCT_CLASS = 2
mxOBJECT_CLASS = 3
mxCHAR_CLASS = 4
mxSPARSE_CLASS = 5
mxDOUBLE_CLASS = 6
mxSINGLE_CLASS = 7
mxINT8_CLASS = 8
mxUINT8_CLASS = 9
mxINT16_CLASS = 10
mxUINT16_CLASS = 11
mxINT32_CLASS = 12
mxUINT32_CLASS = 13

mxArrays = (
    mxCHAR_CLASS,
    mxDOUBLE_CLASS,
    mxSINGLE_CLASS,
    mxINT8_CLASS,
    mxUINT8_CLASS,
    mxINT16_CLASS,
    mxUINT16_CLASS,
    mxINT32_CLASS,
    mxUINT32_CLASS,
    )

def _parse_header(fid, dict):
    correct_endian = (ord('M')<<8) + ord('I')
                 # if this number is read no BS    
    fid.seek(126)  # skip to endian detector
    endian_test = fid.read(1,'int16')
    if (endian_test == correct_endian): openstr = 'n'
    else:  # must byteswap
        if LittleEndian:
            openstr = 'b'
        else: openstr = 'l'
    fid.setformat(openstr)  # change byte-order if necessary
    fid.rewind()    
    dict['__header__'] = fid.fid.read(124).strip(' \t\n\000')
    vers = fid.read(1,'int16')
    dict['__version__'] = '%d.%d' % (vers >> 8, vers & 255)
    fid.seek(2,1)  # move to start of data
    return

def _parse_array_flags(fid):
    # first 8 bytes are always miUINT32 and 8 --- just a check
    dtype, nbytes = fid.read(2,'u')
    if (dtype != miUINT32) or (nbytes != 8):
        raise IOError, "Invalid MAT file. Perhaps a byte-order problem."

    # read array flags.
    rawflags = fid.read(2,'u')
    class_ = rawflags[0] & 255
    flags = (rawflags[0] & 65535) >> 8
    # Global and logical fields are currently ignored
    if (flags & 8): cmplx = 1
    else: cmplx = 0
    if class_ == mxSPARSE_CLASS:
        nzmax = rawflags[1]
    else:
        nzmax = None
    return class_, cmplx, nzmax

def _parse_mimatrix(fid,bytes): 
    dclass, cmplx, nzmax =_parse_array_flags(fid)
    dims = _get_element(fid)[0]
    name = ''.join(asarray(_get_element(fid)[0]).astype('c'))
    if dclass in mxArrays:
        result, unused =_get_element(fid)
        if type == mxCHAR_CLASS:
            result = ''.join(asarray(result).astype('c'))
        else:
            if cmplx:
                imag, unused =_get_element(fid)
                result = result + cast[imag.typecode()](1j) * imag
            result = squeeze(transpose(reshape(result,dims[::-1])))

    elif dclass == mxCELL_CLASS:
        length = product(dims)
        result = zeros(length, PyObject)
        for i in range(length):
            sa, unused = _get_element(fid)
            result[i]= sa
        result = squeeze(transpose(reshape(result,dims[::-1])))
        if rank(result)==0: result = result.toscalar()
        
    elif dclass == mxSTRUCT_CLASS:
        length = product(dims)
        result = zeros(length, PyObject)
        namelength = _get_element(fid)[0]
        # get field names
        names = _get_element(fid)[0]
        splitnames = [names[i:i+namelength] for i in \
                      xrange(0,len(names),namelength)]
        fieldnames = [''.join(asarray(x).astype('c')).strip('\x00')
                              for x in splitnames]
        for i in range(length):
            result[i] = mat_struct()
            for element in fieldnames:
                val,unused = _get_element(fid)
                result[i].__dict__[element] = val
        result = squeeze(transpose(reshape(result,dims[::-1])))
        if rank(result)==0: result = result.toscalar()        

        # object is like a structure with but with a class name
    elif dclass == mxOBJECT_CLASS:
        class_name = ''.join(asarray(_get_element(fid)[0]).astype('c'))
        length = product(dims)
        result = zeros(length, PyObject)
        namelength = _get_element(fid)[0]
        # get field names
        names = _get_element(fid)[0]
        splitnames = [names[i:i+namelength] for i in \
                      xrange(0,len(names),namelength)]
        fieldnames = [''.join(asarray(x).astype('c')).strip('\x00')
                              for x in splitnames]
        for i in range(length):
            result[i] = mat_obj()
            result[i]._classname = class_name
            for element in fieldnames:
                val,unused = _get_element(fid)
                result[i].__dict__[element] = val
        result = squeeze(transpose(reshape(result,dims[::-1])))
        if rank(result)==0: result = result.toscalar()        
         
    elif dclass == mxSPARSE_CLASS:
        rowind, unused = _get_element(fid)
        colind, unused = _get_element(fid)
        res, unused = _get_element(fid)
        if cmplx:
            imag, unused = _get_element(fid)
            res = res + cast[imag.typecode()](1j)*imag
        if have_sparse:
            spmat = scipy.sparse.spmatrix(dims[1],dims[0],typecode=res.typecode())
            spmat.data = res
            # stored with zero-based indices --- spmatrix needs 1 based
            #  indices internally
            spmat.index = [rowind+1, colind+1]
            spmat.nzmax = len(res)
            spmat.lastel = len(res)-1
            result = spmat.transp(inplace=1)
            result = spmat
        else:
            result = (rowind, colind, res)

    return result, name
    
# Return a Python object for the element
def _get_element(fid):

    test = fid.fid.read(1)
    if len(test) == 0:  # nothing left
        raise EOFError
    else:
        fid.rewind(1)
    # get the data tag
    raw_tag = fid.read(1,'u')
    
    # check for compressed
    numbytes = raw_tag >> 16
    if numbytes > 0:  # compressed format
        if numbytes > 4:
            raise IOError, "Problem with MAT file: " \
                  "too many bytes in compressed format."
        dtype = raw_tag & 65535
        el = fid.read(numbytes,miDataTypes[dtype][2],c_is_b=1)
        fid.seek(4-numbytes,1)  # skip padding
        return el, None

    # otherwise parse tag
    dtype = raw_tag
    numbytes = fid.read(1,'u')
    if dtype != miMATRIX:  # basic data type 
        try:
            outarr = fid.read(numbytes,miDataTypes[dtype][2],c_is_b=1)
        except KeyError:
            raise ValueError, "Unknown data type"
        mod8 = numbytes%8
        if mod8:       # skip past padding
            skip = 8-mod8
            fid.seek(skip,1)
        return outarr, None

    # handle miMatrix type
    el, name = _parse_mimatrix(fid,numbytes)
    return el, name

def _loadv5(fid,basename):
    # return a dictionary from a Matlab version 5 file
    # always contains the variable __header__
    dict = {}
    _parse_header(fid,dict)
    var = 0
    while 1:  # file pointer to start of next data
        try:
            var = var + 1
            el, varname = _get_element(fid)
            if varname is None:
                varname = '%s_%04d' % (basename,var)
            dict[varname] = el
        except EOFError:
            break
    return dict

### END MATLAB v5 support #############

def loadmat(name, dict=None, appendmat=1, basename='raw'):
    """Load the MATLAB(tm) mat file.

    If name is a full path name load it in.  Otherwise search for the file
    on the sys.path list and load the first one found (the current directory
    is searched first).

    Both v4 (Level 1.0) and v5 matfiles are supported.

    Inputs:

      name -- name of the mat file (don't need .mat extension if appendmat=1)
      dict -- the dictionary to insert into.  If none the variables will be
              returned in a dictionary.
      appendmat -- non-zero to append the .mat extension to the end of the
                   given filename.
      basename -- for MATLAB(tm) v5 matfiles raw data will have this basename.

    Outputs:

      If dict is None, then a dictionary of names and objects representing the
      stored arrays is returned.
    """

    if appendmat and name[-4:] == ".mat":
        name = name[:-4]
    if os.sep in name:
        full_name = name
        if appendmat:
            full_name = name + ".mat"
    else:
        full_name = None
        junk,name = os.path.split(name)
        for path in sys.path:
            test_name = os.path.join(path,name)
            if appendmat:
                test_name += ".mat"
            try:
                fid = open(test_name,'rb')
                fid.close()
                full_name = test_name
            except IOError:
                pass
        if full_name is None:
            raise IOError, "%s not found on the path." % name

    fid = fopen(full_name,'rb')
    test_vals = fid.fread(4,'byte')

    if not (0 in test_vals):       # MATLAB version 5 format
        fid.rewind()
        thisdict = _loadv5(fid,basename)
        if dict is not None:
            dict.update(thisdict)
            return
        else:
            return thisdict
        

    testtype = struct.unpack('i',test_vals.tostring())
    # Check to see if the number is positive and less than 5000.
    if testtype[0] < 0 or testtype[0] > 4999:
        # wrong byte-order
        if LittleEndian:
            format = 'ieee-be'
        else:
            format = 'ieee-le'
    else:  # otherwise we are O.K.
        if LittleEndian:
            format = 'ieee-le'
        else:
            format = 'ieee-be'

    fid.setformat(format)

    length = fid.size()
    fid.rewind()  # back to the begining

    defnames = []
    thisdict = {}
    while 1:
        if (fid.tell() == length):
            break        
        header = fid.fread(5,'int')
        if len(header) != 5:
            fid.close()
            print "Warning: Read error in file."
            break

        M,rest = divmod(header[0],1000)
        O,rest = divmod(rest,100)
        P,rest = divmod(rest,10)
        T = rest

        if (M > 1):
            fid.close()
            raise ValueError, "Unsupported binary format."
        if (O != 0):
            fid.close()
            raise ValuError, "Hundreds digit of first integer should be zero."

        if (T not in [0,1]):
            fid.close()
            raise ValueError, "Cannot handle sparse matrices, yet."

        storage = {0:'d',1:'f',2:'i',3:'s',4:'w',5:'b'}[P]

        varname = fid.fread(header[-1],'char')[:-1]
        varname = varname.tostring()
        defnames.append(varname)
        numels = header[1]*header[2]
        if T == 0:             # Text data
            data = atleast_1d(fid.fread(numels,storage))
            if header[3]:  # imaginary data
                data2 = fid.fread(numels,storage)
                if data.typecode() == 'f' and data2.typecode() == 'f':
                    new = zeros(data.shape,'F')
                    new.real = data
                    new.imag = data2
                    data = new
                    del(new)
                    del(data2)
            if len(data) > 1:
                data.shape = (header[2], header[1])                
                thisdict[varname] = transpose(squeeze(data))
            else:
                thisdict[varname] = data
        else:
            data = atleast_1d(fid.fread(numels,storage,'char'))
            if len(data) > 1:
                data.shape = (header[2], header[1])
                thisdict[varname] = transpose(squeeze(data))
            else:
                thisdict[varname] = data

    fid.close()
    if dict is not None:
        print "Names defined = ", defnames
        dict.update(thisdict)
    else:
        return thisdict


def savemat(filename, dict):
    """Save a dictionary of names and arrays into the MATLAB-style .mat file.

    This saves the arrayobjects in the given dictionary to a matlab Version 4
    style .mat file.
    """
    storage = {'D':0,'d':0,'F':1,'f':1,'l':2,'i':2,'s':3,'b':5}
    if filename[-4:] != ".mat":
        filename = filename + ".mat"
    fid = fopen(filename,'wb')
    M = not LittleEndian
    O = 0
    for variable in dict.keys():
        var = dict[variable]
        if type(var) is not ArrayType:
            continue
        if var.typecode() == 'c':
            T = 1
        else:
            T = 0
        if var.typecode() == '1':
            var = var.astype('s')
        P = storage[var.typecode()]
        fid.fwrite([M*1000+O*100+P*10+T],'int')

        if len(var.shape) == 1:
            var.shape = (len(var), 1)
        var = transpose(var)

        if len(var.shape) > 2:
            var.shape = (product(var.shape[:-1]), var.shape[-1])

        imagf = var.typecode() in ['F', 'D']
        fid.fwrite([var.shape[1], var.shape[0], imagf, len(variable)+1],'int')
        fid.fwrite(variable+'\x00','char')
        if imagf:
            fid.fwrite(var.real)
            fid.fwrite(var.imag)
        else:
            fid.fwrite(var)
    fid.close()
    return
        
        
            




